#include <std.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include <func/messages.h>

#include "pkgmgrctrl.h"

pkgMgrCtrl::pkgMgrCtrl(int _sigfd, pkgDPkgPM &_PM) : PM(_PM), signalfd(_sigfd)
{
	pipe(in);
	pipe(out);
}

pkgMgrCtrl::~pkgMgrCtrl() 
{}

void pkgMgrCtrl::run()
{
	signal(SIGCHLD, sa_reaper);

	if (fork() == 0)
	{
		dup2(in[0], STDIN_FILENO);
		dup2(out[1], STDOUT_FILENO);
		close(STDERR_FILENO); // Hmm... 

		pkgPackageManager::OrderResult Res = PM.DoInstall();

		if (Res == pkgPackageManager::Failed || _error->PendingError() == true)
		{
			string s, q;
			while (_error->PendingError() == true)
			{	
				_error->PopMessage(q);
				s += "  * ";
				s += q;
				s += "\n";
			}
			TextMessage(signalfd, MsgId_PkgMgr_Warning, "\nPackage manager (dpkg) failed: \n\n%s", s.c_str());
	
		}
		else if (Res == pkgPackageManager::Completed)
			TextMessage(signalfd, MsgId_PkgMgr_Warning, "\nPackage manager (dpkg) succeeded.\n");

		MessageSend(signalfd, MsgId_PkgMgr_Stop, 0, 0);
	
		exit(0);
	}

	block();
}

void pkgMgrCtrl::block()
{
	fd_set Rset;
	int highfd;
	char buf[BUFSIZ];
	long unsigned int len;
	MsgId mt;

	if (signalfd > out[0])
		highfd = signalfd;
	else
		highfd = out[0];
		
	while (1) {
		FD_ZERO(&Rset);
		FD_SET(signalfd, &Rset);
		FD_SET(out[0], &Rset);
		if (select(highfd + 1, &Rset, NULL, NULL, NULL) > 0) {
			if (FD_ISSET(signalfd, &Rset)) {
				MessageReceive(signalfd, mt, buf, len);
				if (mt == MsgId_PkgMgr_Input) {
					INFO(INFO_VERBOSE, "PKGMGR INPUT len = %d\n", len);
					write(in[1], buf, len);
				} else if (mt == MsgId_PkgMgr_Stop) {
					INFO(INFO_DEBUG, "Main thread told us that the package install is complete -- I obey\n");
					/* Woohoo, package manager is done !!!! 
					   Mr sulu, get us out of here, MAXIMUM Warp! */
					break;
				}
			}
			else if (FD_ISSET(out[0], &Rset)) {
				len = read(out[0], buf, BUFSIZ);
				MessageSend(signalfd, MsgId_PkgMgr_Output, buf, len);
				INFO(INFO_VERBOSE, "PKGMGR OUTPUT len = %d\n", len);
			}
		}
	}
}

void pkgMgrCtrl::sa_reaper(int sig)
{
	int status;
	// Wait for the child to go away
	wait(&status);
}


