/* $Id: config.cc,v 1.21 2001/05/14 15:07:02 ltd Exp $ */
#include <stdio.h>
#include <fstream>
#include <vector>
#include <string.h>
#include <std.h>
#include <parsers/strutl.h>
#include <interface/deityui.h>
#include "config.h"

Config _Conf;

Config::param Config::valid_params[] =
{
	{ "deity.fix_broken_packages",		_TRUE,		false },
	{ "deity.sorting",			"3/4/1/2",	false },
	{ "deity.tree.headers.show",		_TRUE,		false },
	{ "deity.tree.headers.expansion",	"0",		false },
	{ "deity.tree.verbosity",		"high",		false },
	{ "deity.acquire.embedded",		_FALSE,		false },
	{ "deity.acquire.captris",		_FALSE,		false },
	{ "deity.acquire.captris.score",	"0",		false },
	{ "deity.description.show",		_FALSE,		false },
	{ "deity.search.expand_matched_item_parents",	_TRUE,	false },
	{ "deity.search.new_search_discards_old_items",	_TRUE,	false },
	{ "deity.curses.color.default",		"white/black",	false },
	{ "deity.curses.color.cursor",		"%reverse",	false },
	{ "deity.curses.color.depends",	"%bold,black/cyan",	false },
	{ "deity.curses.color.tree",	"%bold,green/black",	false },
	{ "deity.curses.color.header",	"%bold,white/black",	false },
	{ "deity.curses.color.curstate.kept",	"white/black",	false },
	{ "deity.curses.color.curstate.installed", "%bold,white/black",	false },
	{ "deity.curses.color.curstate.upgraded",	"green/black",	false },
	{ "deity.curses.color.newstate.install", "%bold,yellow/black",	false },
	{ "deity.curses.color.newstate.upgrade", "%bold,cyan/black",	false },
	{ "deity.curses.color.newstate.remove",	"red/black",	false },
	{ "deity.curses.color.newstate.purge",	"%bold,red/black",	false },
	{ "deity.curses.color.matched_by_search", "black/red",	false },
	{ "deity.curses.color.pkgmgr", "white/black",	false },
	{ "deity.gtk.color.cursor",		"00000 00000 00000",	false }, // black
	{ "deity.gtk.color.curstate.kept",	"65280 65280 65280",	false }, // white
	{ "deity.gtk.color.curstate.upgraded",	"19968 60928 37888",	false }, // light sea green
	{ "deity.gtk.color.curstate.not_upgraded", "00000 65280 00000",	false }, // strong green1
	{ "deity.gtk.color.newstate.install",	"65280 60416 35584",	false }, // light yellow
	{ "deity.gtk.color.newstate.upgrade",	"34560 52736 64000",	false }, // light sky blue
	{ "deity.gtk.color.newstate.remove",	"65280 25344 18176",	false }, // light red
	{ "deity.gtk.color.newstate.purge",	"65280 00000 00000",	false }, // strong red1
	{ "deity.gtk.font.packages",		"-b&h-lucida-medium-r-normal-*-*-100-*-*-p-*-iso8859-1", false },
	{ "deity.gtk.font.description",		"-b&h-lucidatypewriter-medium-r-normal-sans-10-*-100-100-m-*-iso8859-1", false },
	{ "deity.gtk.tree.columnoffset.1", "0.20", false },
	{ "deity.gtk.tree.columnoffset.2", "0.27", false },
	{ "deity.gtk.tree.columnoffset.3", "0.34", false },
	{ "deity.gtk.tree.columnoffset.4", "0.44", false },
	{ "deity.gtk.tree.columnoffset.5", "0.54", false },
	{ "deity.gtk.window.main.xsize",		"800", false },
	{ "deity.gtk.window.main.ysize",		"600", false },
	{ "deity.gtk.window.filterselect.xsize",		"400", false },
	{ "deity.gtk.window.filterselect.ysize",		"300", false },
	{ NULL,	NULL,	false }
};

bool Config::Write(void)
{
	string filename;
	
	filename += getenv("HOME");
	filename += "/";
	filename += ".deityrc";

	ofstream CCHandle(filename.c_str());

	if (!CCHandle.is_open())
		return false;

	for (struct param *p=valid_params; p->param_name != NULL; p++)
	{
		CCHandle << p->param_name << "=" << p->param_value << endl;
	}

	return true;
}

bool Config::Init(void)
{
	string filename;
	
	filename += getenv("HOME");
	filename += "/";
	filename += ".deityrc";

	ifstream CCHandle(filename.c_str());

	if (!CCHandle.is_open())
		return false;

	string buffer;

	char buf[BUFSIZ], *x;

	while (1)
	{
		CCHandle.getline(buf, BUFSIZ-1, '\n');

		if (CCHandle.eof() == true)
			break;

		if ((x=strchr(buf, '\n'))) 
			*x=0;

		for(x=buf; *x != '=' && *x != 0; x++);

		if (*x == 0)
		{
			printf("initconfig() parse error: %s (must be name=value pair)\n", buf);
			continue;
		}

		*x = 0;

		x++;

		if (*x == 0)
		{
			printf("initconfig() parse error: %s (must be name=value pair)\n", buf);
			continue;
		}

		if (IsValid(buf) == false)
		{
			printf("initconfig(): invalid option: %s (do not edit the config yourself)\n", buf);
			continue;
		}

		Set(buf, x);
	}

	return true;
}

char *Config::Get(char *name)
{
	for (struct param *p=valid_params; p->param_name != NULL; p++)
	{
		if (strcmp(name, p->param_name) == 0)
			return p->param_value;
	}

	if (CaptUI)
		delete CaptUI;

	fprintf(stderr, "Config Item '%s' Referenced (But not defined)\n", name);

	exit(-10);

	return NULL;
}

bool Config::Set(char *name, char *val)
{
	for (struct param *p=valid_params; p->param_name != NULL; p++)
	{
		if (strcmp(name, p->param_name) == 0)
		{
			if (p->override == true)
				free(p->param_value);
			else
				p->override = true;
			p->param_value = strdup(val);
			return true;
		}
	}

	return false;
}

bool Config::IsValid(char *name)
{
	for (struct param *p=valid_params; p->param_name != NULL; p++)
	{
		if (strcmp(name, p->param_name) == 0)
			return true;
	}

	return false;
}

bool Config::GetSplit(char *name, char *div, vector<string> & ret)
{
	char *val = Get(name);

	if (val == NULL)
		return false;

	ret.resize(0);

	for (char *p = sane_strtok(name, div); p != NULL; p = sane_strtok(NULL, div))
	{
		ret.resize(ret.size()+1);
		ret[ret.size()-1] = p;
	}

	return true;
}


