/* $Id: keys.cc,v 1.27 2001/05/23 22:41:12 ltd Exp $ */
#include <std.h>
#include <ops.h>
#ifdef HAVE_LIBGPM
#include <gpm.h>
#endif 
#include <func/apt.h>
#include <func/config.h>
#include <func/pkgchecks.h>
#include <func/tree.h>
#include <func/dependency.h>
#include <interface/deityui.h>
#include <backend/acquire.h>
#include <list/screens.h>
#include <list/filters.h>
#include <ncurses.h> // just for the KEY constants...
#include "keys.h"

#define SECTION(name) \
	{ -1, NULL, NULL, NULL, "$1#" ## name ## "$!" }
#define SCR_KEY(key, func, kdesc, desc) \
	{ key, NULL, &Screen::func, kdesc, desc }
#define DEF_KEY(key, func, kdesc, desc) \
	{ key, DefaultKeys::func, NULL, kdesc, desc }
#define RES_KEY(key, func, kdesc, desc) \
	{ key, RestrictedKeys::func, NULL, kdesc, desc }
#define FN_KEY(key, func, kdesc, desc) \
	{ key, func, NULL, kdesc, desc }
#define END_KEYS \
	{ 0, NULL, NULL, NULL, NULL }

KeyHandler::List DefaultKeys::Bindings[] =
{
	SECTION("Console APT Help"),
	DEF_KEY('q', quit, "q", "Quit the program"),
	SCR_KEY('?', HelpDialog, "?", "Display this help screen"),
	SCR_KEY('h', HelpDialog, "h", "Display this help screen"),
	SECTION("Screen Management"),
	DEF_KEY('!', screen_create_new, "!", "Create new listing screen"),
	SECTION("List Toggles"),
	DEF_KEY('\t', depend_toggle, "Tab", "Show package dependencies"),
	DEF_KEY('\n', depend_toggle, "Enter", "Show package dependencies"),
	DEF_KEY('d', descwin_toggle, "d", "Show package description"),
	DEF_KEY('V', viewmode_toggle, "V", "Toggle verbose view mode"),
	SCR_KEY('s', SortSelectMethod, "s", "Change list sort method(s)"),
	DEF_KEY('/', search, "/", "Search through the list"),
	DEF_KEY(18, reset_markers, "Ctrl-R", "Reset items marked by search"),
	SCR_KEY('n', SeekNextMatched, "n", "Move to next search match"),
	SECTION("List Movement"),
	SCR_KEY(KEY_UP, ArrowUp, "Up", "Move to the previous package"),
	SCR_KEY(KEY_DOWN, ArrowDown, "Down", "Move to the next package"),
	SCR_KEY('k', ArrowUp, "k", "Move to the previous package"),
	SCR_KEY('j', ArrowDown, "j", "Move to the next package"),
	SCR_KEY('^', SeekStart, "^", "Move to the start of the list"),
	SCR_KEY('g', SeekStart, "g", "Move to the start of the list"),
	SCR_KEY('$', SeekEnd, "$$", "Move to the end of the list"),
	SCR_KEY('G', SeekEnd, "G", "Move to the end of the list"),
	SCR_KEY(KEY_HOME, SeekStart, "Home", "Move to the list start"),
	SCR_KEY(KEY_END, SeekEnd, "End", "Move to the list end"),
	SCR_KEY(2, PageUp, "CTRL-F", "Move up one screens length"),
	SCR_KEY(6, PageDown, "CTRL-B", "Move down one screens length"),
	SCR_KEY(KEY_PPAGE, PageUp, "PageUp", "Move up one screens length"),
	SCR_KEY(KEY_NPAGE, PageDown, "PageDown", "Move down one screens length"),
	SCR_KEY(KEY_LEFT, ArrowLeft, "Left", "Scroll description left (Verbose only)"),
	SCR_KEY(KEY_RIGHT, ArrowRight, "Right", "Scroll description right (Verbose only)"),
	SCR_KEY('[', ScrollDescUp, "[", "Scroll package description UP"),
	SCR_KEY(']', ScrollDescDown, "]", "Scroll package description DOWN"),
	SCR_KEY('<', DividerPrev, "<", "Move to the PREVIOUS package divider"),
	SCR_KEY('>', DividerNext, ">", "Move to the NEXT package divider"),
	SECTION("List Filters"),
	FN_KEY('f', filter_add_single, "f", "Add new filter"),
	FN_KEY('r', filter_revoke_all, "r", "Revoke all filters"),
	FN_KEY('l', filter_display_all, "l", "Display filter list"),
	FN_KEY('v', filter_change_view, "v", "Reverse filters"),
	SECTION("Package Modifiers"),
	DEF_KEY('t', iterate_version, "t", "Iterate through available versions"),
	DEF_KEY('\\', pkg_reinstall, "\\", "Mark or unmark for RE-install"),
	DEF_KEY('+', pkg_install, "+", "Mark or unmark for install/upgrade"),
	DEF_KEY(' ', pkg_install, "Space", "Mark or unmark for install/upgrade"),
	DEF_KEY('-', pkg_remove, "-", "Mark or unmark for deletion"),
	DEF_KEY('_', pkg_purge, "_", "Mark or unmark for purgation (deletes config files)"),
	DEF_KEY('U', all_upgrade, "U", "Perform a global package upgrade"),
	DEF_KEY('D', dist_upgrade, "D", "Perform a debian distribution upgrade"),
	SECTION("Install Functions"),
	DEF_KEY('c', complete_pending, "c", "Complete pending install/removals"),
	DEF_KEY(KEY_BACKSPACE, complete_pending, "Backspace", "Complete pending install/removals"),
	DEF_KEY('u', update_lists, "u", "Update available package lists"),
	DEF_KEY('R', reread_cache, "R", "Reinitialize the apt cache"),
	END_KEYS
};

KeyHandler::List RestrictedKeys::Bindings[] =
{
	SECTION("Console APT Help"),
	SCR_KEY('?', HelpDialog, "?", "Display this help screen"),
	SCR_KEY('h', HelpDialog, "h", "Display this help screen"),
	RES_KEY('q', abort, "q", "Abort package acquisition"),
	RES_KEY('c', complete, "c", "Continue with package acquisition"),
	SECTION("List Toggles"),
	DEF_KEY('\t', depend_toggle, "Tab", "Show package dependencies"),
	DEF_KEY('\n', depend_toggle, "Enter", "Show package dependencies"),
	DEF_KEY('d', descwin_toggle, "d", "Show package description"),
	DEF_KEY('V', viewmode_toggle, "V", "Toggle verbose view mode"),
	DEF_KEY('/', search, "/", "Search through the list"),
	DEF_KEY( 18, reset_markers, "Ctrl-R", "Reset items marked by search"),
	SCR_KEY('n', SeekNextMatched, "n", "Move to next search match"),
	SECTION("List Filters"),
	FN_KEY('f', filter_add_single, "f", "Add new filter"),
	FN_KEY('r', filter_revoke_all, "r", "Revoke all filters"),
	FN_KEY('l', filter_display_all, "l", "Display filter list"),
	FN_KEY('v', filter_change_view, "v", "Invert filter polarity"),
	SECTION("Package Movement"),
	SCR_KEY(KEY_UP, ArrowUp, "Up", "Move to the previous package"),
	SCR_KEY(KEY_DOWN, ArrowDown, "Down", "Move to the next package"),
	SCR_KEY('k', ArrowUp, "k", "Move to the previous package"),
	SCR_KEY('j', ArrowDown, "j", "Move to the next package"),
	SCR_KEY('^', SeekStart, "^", "Move to the start of the list"),
	SCR_KEY('g', SeekStart, "g", "Move to the start of the list"),
	SCR_KEY('$', SeekEnd, "$$", " Move to the end of the list"),
	SCR_KEY('G', SeekEnd, "G", "Move to the end of the list"),
	SCR_KEY(KEY_HOME, SeekStart, "Home", "Move to the list start"),
	SCR_KEY(KEY_END, SeekEnd, "End", "Move to the list end"),
	SCR_KEY(2, PageUp, "CTRL-F", "Move up one screens length"),
	SCR_KEY(6, PageDown, "CTRL-B", "Move down one screens length"),
	SCR_KEY(KEY_PPAGE, PageUp, "PageUp", "Move up one screens length"),
	SCR_KEY(KEY_NPAGE, PageDown, "PageDown", "Move down one screens length"),
	SCR_KEY(KEY_LEFT, ArrowLeft, "Left", "Scroll description left (Verbose only)"),
	SCR_KEY(KEY_RIGHT, ArrowRight, "Right", "Scroll description right (Verbose only)"),
	SCR_KEY('[', ScrollDescUp, "[", "Scroll package description UP"),
	SCR_KEY(']', ScrollDescDown, "]", "Scroll package description DOWN"),
	END_KEYS
};

bool KeyHandler::DispatchKey(int Keypress)
{
	List *p;

	for (p = klptr; p->key != 0; p++)
	{
		if ((p->handlefunc == NULL) && (p->scrfunc == 0))
			continue;

		if (Keypress == p->key)
		{
			if (p->handlefunc)
				p->handlefunc(&screen);
			else
				(screen.*(p->scrfunc))();
			break;
		}
	}

	if (isdigit(Keypress))
	{
		Screen *P = CaptUI->scrlst.Head();
		int i = 49; // '1'
		while (i < Keypress && P->next != NULL)
		{
			i++;
			P = P->next;
		}
		CaptUI->scrlst.SetCurrent(P);   // Sanity Checked --Pat
		P->Redraw();
		return true;
	}

	// For the menu

	switch (Keypress)
	{
		// Operation
		case 'O':
			screen.ActivateMenu(Screen::M_Functions);
		break;
		// Movement
		case 'M':
			screen.ActivateMenu(Screen::M_Movement);
			break;
		// Toggles
		case 'T':
			screen.ActivateMenu(Screen::M_Toggles);
			break;
		// Alter
		case 'A':
			screen.ActivateMenu(Screen::M_Modifiers);
			break;
		// Filters
		case 'F':
			screen.ActivateMenu(Screen::M_Filters);
			break;
		// Sorting
		case 'S':
			screen.ActivateMenu(Screen::M_Sorting);
			break;
	}

	return true;
}

// ...

void DefaultKeys::iterate_version(Screen *screen)
{
	pkgCache::VerIterator V = screen->cs->Version();

	V++;

	if (V.end() == true)
		V = screen->cs->TargetPkg.VersionList();
	
	(*Cache)->SetCandidateVersion(V);

	//screen->cs->TargetVer = V;
	
	screen->RedrawTree();
}

void DefaultKeys::viewmode_toggle(Screen *screen)
{
	if (screen->mode == Screen::Terse)
	{
		screen->mode = Screen::Verbose;
		_Conf.Set("deity.tree.verbosity", "high");
	}
	else
	{
		screen->mode = Screen::Terse;
		_Conf.Set("deity.tree.verbosity", "low");
	}

	screen->Redraw();

	return;
}

void DefaultKeys::screen_create_new(Screen *screen)
{
  Screen *s = CaptUI->scrlst.add();
	s->BoundsReset();
	s->Redraw();
}

/*
 * Wrapper for the online search option: Search through the current
 * screen's package list.
 *
 *  The user can enter a blank string to repeat the last search.
 */
void DefaultKeys::search(Screen *screen)
{
	char *buf = (char *)malloc(256);
	unsigned long options = 0;

	screen->SearchEntry(&buf, &options);
		    
	if (*buf)
	{   
		if (_Conf.GetBool("deity.search.new_search_discards_old_items"))
			screen->Packages->RemoveState(PackageTree::Item::Matched);

		if (_Conf.GetBool("deity.search.expand_matched_item_parents"))
			screen->Packages->CollapseDividers();

		if (screen->Packages->Locate(buf, options) > 0)
		{
			screen->Packages->RecalcLength();
			screen->SeekNextMatched();
			screen->BoundsReset();
			screen->Redraw();
		}
		else
			if (_Conf.GetBool("deity.search.expand_matched_item_parents"))
			{
				screen->Packages->RecalcLength();
				screen->BoundsReset();
				screen->Redraw();
			}
			
	}

	free(buf);

	return;
}

void DefaultKeys::descwin_toggle(Screen *screen)
{
	if (screen->IsDescWindowShown() == false)
	{
		screen->DescWindowShow();
		_Conf.Set("deity.ui.description.show", _TRUE);
	}
	else
	{
		screen->DescWindowHide();
		_Conf.Set("deity.ui.description.show", _FALSE);
	}

	screen->RedrawTree();

	return;
}

void DefaultKeys::depend_toggle(Screen *screen)
{
	if (screen->cs->IsDivider())
	{
		if (screen->cs->State & PackageTree::Item::Expanded)
			(int)screen->cs->State &= ~ PackageTree::Item::Expanded;
		else
			(int)screen->cs->State |= PackageTree::Item::Expanded;

		screen->Packages->RecalcLength();
	}
	else if (screen->cs->IsDepend() && screen->cs->Version().end() == true)
		return;
	else
	{
		if (screen->cs->IsExpanded() == true)
			depend_tree_destruct(screen->cs, *screen->Packages);
		else
			depend_tree_expand(screen->cs, *screen->Packages);
	}

	screen->BoundsReset();
	screen->RedrawTree();

	return;
}

void DefaultKeys::reread_cache(Screen *screen)
{
	Cache->Reinit();
	screen->Redraw();
	return;
}

void DefaultKeys::update_lists(Screen *screen)
{
	if (getuid() != 0)
	{
		CaptUI->ShowDialog("Error", "\nRoot privileges are required to update the package lists \n");
		screen->Redraw();
		return;
	}
	
	pkgAcquireBackend *Fetcher = new pkgAcquireBackend(CaptUI);
	int fd = Fetcher->AcquireUpdate();
	CaptUI->WaitForFetch(Fetcher, fd);

	return;
}

void DefaultKeys::all_upgrade(Screen *screen)
{
	if (pkgAllUpgrade(*Cache) == false)
		capt_show_broken(*Cache);

	screen->Redraw();

	return;
}

void DefaultKeys::dist_upgrade(Screen *screen)
{
	if (pkgDistUpgrade(*Cache) == false)
		capt_show_broken(*Cache);

	screen->Redraw();

	return;
}

void DefaultKeys::pkg_remove(Screen *screen)
{
	if (screen->cs->Type != PackageTree::Item::Pkg)
		return;

	pkgCache::PkgIterator I = screen->cs->TargetPkg;
	
	if (I.CurrentVer().end() == false)
	{
		if ((*Cache)[I].Delete() == true)
			(*Cache)->MarkKeep(I);
		else
			TryRemove(I);
	}

	screen->ArrowDown();
	screen->Redraw();

	return;
}

void DefaultKeys::pkg_purge(Screen *screen)
{
	if (screen->cs->Type != PackageTree::Item::Pkg)
		return;

	pkgCache::PkgIterator I = screen->cs->TargetPkg;

	if (I.CurrentVer().end() == false)
	{
		if ((*Cache)[I].Delete() == true)
			(*Cache)->MarkKeep(I);
		else
			TryRemove(I, true);
	}

	screen->ArrowDown();
	screen->Redraw();

	return;
}

void DefaultKeys::pkg_install(Screen *screen)
{
	if (screen->cs->Type != PackageTree::Item::Pkg)
		return;

	pkgCache::PkgIterator I = screen->cs->TargetPkg;
	
	if (I.CurrentVer().end() == true || (*Cache)[I].Status < 0 || (*Cache)[I].Status == 1)
	{
		if ((*Cache)[I].Install() == true)
			(*Cache)->MarkKeep(I);
		else
			TryInstall(I);
	}

	screen->ArrowDown();
	screen->Redraw();

	return;
}

void DefaultKeys::pkg_reinstall(Screen *screen)
{
	if (screen->cs->Type != PackageTree::Item::Pkg)
		return;

	pkgCache::PkgIterator I = screen->cs->TargetPkg;
	
	if (I.CurrentVer().end() == false)
	{
		if ((*Cache)[I].Install() == true)
			(*Cache)->MarkKeep(I);
		else
			TryInstall(I, true);
	}

	screen->ArrowDown();
	screen->Redraw();

	return;
}

void DefaultKeys::pkg_hold(Screen *screen)
{
	if (screen->cs->Type != PackageTree::Item::Pkg)
		return;

	(*Cache)->MarkKeep((pkgCache::PkgIterator &) *screen->cs);
}

void DefaultKeys::complete_pending(Screen *screen)
{
	if ((*Cache)->DelCount() == 0 && (*Cache)->InstCount() == 0 && (*Cache)->BadCount() == 0)
		return;

	if (getuid() != 0)
	{
		CaptUI->ShowDialog("Error", "\nRoot privileges are required to install, upgrade, or remove packages\n");
		screen->Redraw();
		return;
	}

	if ((*Cache)->BrokenCount() != 0) 
	{                           
		if (_Conf.GetBool("deity.fix_broken_packages"))
		{
			Cache->Resolver().InstallProtect();
			if (Cache->Resolver().Resolve(true) == false)
				_error->Discard();
		}
		else
		{
			capt_show_broken(*Cache);
			return;
		}
	}   

	Screen *confirmscr = CaptUI->scrlst.add(Screen::PkgInit, Screen::InstallConf);
	
	confirmscr->BoundsReset();
	confirmscr->Redraw();

	CaptUI->Modal(confirmscr);

	return;
}

void DefaultKeys::quit(Screen *screen)
{
	deity_clean_exit();
	exit(0);
}

void DefaultKeys::reset_markers(Screen *screen)
{
	screen->Packages->RemoveState(PackageTree::Item::Matched);
	screen->Redraw();

	return;
}

void DefaultKeys::changed_packages(Screen *screen)
{
	Screen *confirmscr = CaptUI->scrlst.add(Screen::noPkgInit);
	confirmscr->Packages = new PackageTree(confirmscr, PackageTree::ModPkgs);
	confirmscr->BoundsReset();

	screen->Redraw();
}

// ... //

void RestrictedKeys::complete(Screen *screen)
{
	pkgAcquireBackend *Fetcher = new pkgAcquireBackend(CaptUI);
	int fd = Fetcher->AcquireRun();
	CaptUI->WaitForFetch(Fetcher, fd);
	CaptUI->scrlst.remove(screen);
}

void RestrictedKeys::abort(Screen *screen)
{
	CaptUI->scrlst.remove(screen);
	CaptUI->scrlst.SetCurrent(CaptUI->scrlst.Head());   // Sanity Checked --Pat
	CaptUI->scrlst.Current()->Redraw();
}

