/* $Id: cursesmnu.cc,v 1.9 2001/03/24 08:08:32 ltd Exp $ */
#include <std.h>
#include <list/screens.h>
#include <list/sorting.h>
#include "colors.h"
#include <func/keys.h>
#include "cursesui.h"
#include "cursesmnu.h"

#define _ALG ((COLS)/6)

#define FN_KEY(type, func, desc) \
	{ ::Screen::type, ::func, NULL, desc }
#define DEF_KEY(type, func, desc) \
	{ ::Screen::type, DefaultKeys::func, NULL, desc }
#define SCR_KEY(type, func, desc) \
	{ ::Screen::type, NULL, &Screen::func, desc }


CursesUI::MenuDrop::menu_item_list CursesUI::MenuDrop::mi[] =
{

	FN_KEY(M_Filters, filter_add_single, "Add new filter"),
	FN_KEY(M_Filters, filter_revoke_all, "Remove active filters"),
	FN_KEY(M_Filters, filter_display_all, "Display filter list"),
	FN_KEY(M_Filters, filter_change_view, "Reverse filter polarity"),

	DEF_KEY(M_Modifiers, pkg_install, "(Un)Mark pkg for install/upgrade"),
	DEF_KEY(M_Modifiers, pkg_remove, "(Un)Mark pkg for remove"),
	DEF_KEY(M_Modifiers, pkg_purge, "(Un)Mark pkg for purge"),
	DEF_KEY(M_Modifiers, all_upgrade, "Perform a global package upgrade"),
	DEF_KEY(M_Modifiers, dist_upgrade, "Perform a debian distribution upgrade"),

	SCR_KEY(M_Functions, HelpDialog, "Display key bindings" ),
	DEF_KEY(M_Functions, complete_pending, "Complete pending install/removals"),
	DEF_KEY(M_Functions, update_lists, "Update available package lists"),
	DEF_KEY(M_Functions, reread_cache, "Reinitialize the apt cache"),
	SCR_KEY(M_Functions, SortSelectMethod, "Set tree sorting method(s)"),
	DEF_KEY(M_Functions, search, "Search through the package list"),
	DEF_KEY(M_Functions, reset_markers, "Reset items marked by search"),
	SCR_KEY(M_Functions, SeekNextMatched, "Move to next search match"),
	DEF_KEY(M_Functions, quit, "Quit program"),

	DEF_KEY(M_Toggles, depend_toggle, "Package Dependencies"),
	DEF_KEY(M_Toggles, descwin_toggle, "Description Window"),
	DEF_KEY(M_Toggles, viewmode_toggle, "Verbosity of Tree View"),

	SCR_KEY(M_Movement, ArrowDown, "Move to the next package"),
	SCR_KEY(M_Movement, ArrowUp, "Move to the previous package"),
	SCR_KEY(M_Movement, SeekStart, "Move to the start of the list"),
	SCR_KEY(M_Movement, SeekEnd, "Move to the end of the list"),
	SCR_KEY(M_Movement, PageDown, "Move down one screen length"),
	SCR_KEY(M_Movement, PageUp, "Move up one screen length"),
	SCR_KEY(M_Movement, ArrowLeft, "Scroll description left (Verbose only)"),
	SCR_KEY(M_Movement, ArrowRight, "Scroll description right (Verbose only)"),
	SCR_KEY(M_Movement, ScrollDescUp, "Scroll package description UP"),
	SCR_KEY(M_Movement, ScrollDescDown, "Scroll package description DOWN"),
	SCR_KEY(M_Movement, DividerPrev, "Move to the PREVIOUS package divider"),
	SCR_KEY(M_Movement, DividerNext, "Move to the NEXT package divider"),
	{ (::Screen::Menu) -1,    NULL, NULL, NULL }
};

CursesUI::MenuDrop::MenuDrop(Menu M, Screen &scr) : CursesUI::Dialog(scr)
{
	Current = M;

	geometry(-1, -1, 1, _ALG * M);
	if (M == ::Screen::M_Sorting)
		sorting();
	else
		import_menu_items(M);
	initialize();
	Window->ChangeColor(CursesUI::_clrsys->regpair(COLOR_BLUE, COLOR_BLUE)|A_BOLD, CursesUI::_clrsys->regpair(COLOR_GREEN, COLOR_BLUE)|A_BOLD);
	drawall();
	Window->Refresh();
	select();
}

void CursesUI::MenuDrop::move_toward(Menu M)
{
	Current = M;

	deinitialize();
	geometry(-1, -1, 1, _ALG * M);
	clear_menu_items();
	if (M == ::Screen::M_Sorting)
		sorting();
	else
		import_menu_items(M);
	screen.RefreshCurrent();
	initialize();
	Window->ChangeColor(CursesUI::_clrsys->regpair(COLOR_BLUE, COLOR_BLUE)|A_BOLD, CursesUI::_clrsys->regpair(COLOR_GREEN, COLOR_BLUE)|A_BOLD);
	drawall();
	Window->Refresh();
}

void CursesUI::MenuDrop::clear_menu_items()
{
	free_memory();
}

bool CursesUI::MenuDrop::external_key_handler(int key)
{
	if (key == KEY_RIGHT)
	{
		switch (Current)
		{
			case ::Screen::M_Functions:
				move_toward(::Screen::M_Movement);
				break;
			case ::Screen::M_Movement:
				move_toward(::Screen::M_Toggles);
				break;
			case ::Screen::M_Toggles:
				move_toward(::Screen::M_Modifiers);
				break;
			case ::Screen::M_Modifiers:
				move_toward(::Screen::M_Filters);
				break;
			case ::Screen::M_Filters:
				move_toward(::Screen::M_Sorting);
				break;
			case ::Screen::M_Sorting:
				move_toward(::Screen::M_Functions);
				break;
		}
	}
	else if (key == KEY_LEFT)
	{
		switch (Current)
		{
			case ::Screen::M_Functions:
				move_toward(::Screen::M_Sorting);
				break;
			case ::Screen::M_Movement:
				move_toward(::Screen::M_Functions);
				break;
			case ::Screen::M_Toggles:
				move_toward(::Screen::M_Movement);
				break;
			case ::Screen::M_Modifiers:
				move_toward(::Screen::M_Toggles);
				break;
			case ::Screen::M_Filters:
				move_toward(::Screen::M_Modifiers);
				break;
			case ::Screen::M_Sorting:
				move_toward(::Screen::M_Filters);
				break;
		}
	} 
	else if (key == '\n')
	{
		if (Selection()->handler != NULL)
		{
			(*Selection()->handler)(key);
			return false;
		}
	}
	else if (key == ' ')
		if (Selection()->handler != NULL)
		{
			(*Selection()->handler)(key);
			clear_menu_items();
			sorting();
			screen.RefreshCurrent();
			complete_redraw();
		}

	return true;
}

void CursesUI::MenuDrop::import_menu_items(Menu M)
{
	// TODO: fix this....
	for (menu_item_list *P = mi; P->caption != NULL; P++)
		if (M == P->menu) {
			if (P->binding)
				addline(P->caption, P->binding);
			else if (P->uifunc)
				addline(P->caption,  P->uifunc);
		}
}

void CursesUI::MenuDrop::sort_toggle_section(::Screen *screen)
{
	SortOrderList::SortIterator S = screen->Packages->Sorting->ListFind(SortOrderList::Section);
	if (S.end() == true)
		screen->Packages->Sorting->Push(SortOrderList::Section);
	else
		screen->Packages->Sorting->remove(S);

	screen->Packages->reinitialize();
	screen->BoundsReset();
	screen->RedrawTree();
}
			
void CursesUI::MenuDrop::sort_toggle_priority(::Screen *screen)
{
	SortOrderList::SortIterator S = screen->Packages->Sorting->ListFind(SortOrderList::Priority);
	if (S.end() == true)
		screen->Packages->Sorting->Push(SortOrderList::Priority);
	else
		screen->Packages->Sorting->remove(S);

	screen->Packages->reinitialize();
	screen->BoundsReset();
	screen->RedrawTree();
}

void CursesUI::MenuDrop::sort_toggle_state(::Screen *screen)
{
	SortOrderList::SortIterator S = screen->Packages->Sorting->ListFind(SortOrderList::Status);
	if (S.end() == true)
		screen->Packages->Sorting->Push(SortOrderList::Status);
	else
		screen->Packages->Sorting->remove(S);

	screen->Packages->reinitialize();
	screen->BoundsReset();
	screen->RedrawTree();
}

void CursesUI::MenuDrop::sort_toggle_pkgname(::Screen *screen)
{
	SortOrderList::SortIterator S = screen->Packages->Sorting->ListFind(SortOrderList::PkgName);
	if (S.end() == true)
		screen->Packages->Sorting->Push(SortOrderList::PkgName);
	else
		screen->Packages->Sorting->remove(S);

	screen->Packages->reinitialize();
	screen->BoundsReset();
	screen->RedrawTree();
}

void CursesUI::MenuDrop::sort_toggle_size(::Screen *screen)
{
	SortOrderList::SortIterator S = screen->Packages->Sorting->ListFind(SortOrderList::Size);
	if (S.end() == true)
		screen->Packages->Sorting->Push(SortOrderList::Size);
	else
		screen->Packages->Sorting->remove(S);

	screen->Packages->reinitialize();
	screen->BoundsReset();
	screen->RedrawTree();
}

void CursesUI::MenuDrop::sort_toggle_instsize(::Screen *screen)
{
	SortOrderList::SortIterator S = screen->Packages->Sorting->ListFind(SortOrderList::Instsize);
	if (S.end() == true)
		screen->Packages->Sorting->Push(SortOrderList::Instsize);
	else
		screen->Packages->Sorting->remove(S);

	screen->Packages->reinitialize();
	screen->BoundsReset();
	screen->RedrawTree();
}

void CursesUI::MenuDrop::sort_toggle_suite(::Screen *screen)
{
	SortOrderList::SortIterator S = screen->Packages->Sorting->ListFind(SortOrderList::Suite);
	if (S.end() == true)
		screen->Packages->Sorting->Push(SortOrderList::Suite);
	else
		screen->Packages->Sorting->remove(S);

	screen->Packages->reinitialize();
	screen->BoundsReset();
	screen->RedrawTree();
}

void CursesUI::MenuDrop::sorting(void)
{
	if (screen.Packages->Sorting->Exists(SortOrderList::Section))
		addline("[$3#X$!] Section", &sort_toggle_section);
	else
		addline("[ ] Section", &sort_toggle_section);

	if (screen.Packages->Sorting->Exists(SortOrderList::Priority))
		addline("[$3#X$!] Priority", &sort_toggle_priority);
	else
		addline("[ ] Priority", &sort_toggle_priority);

	if (screen.Packages->Sorting->Exists(SortOrderList::Status))
		addline("[$3#X$!] State", &sort_toggle_state);
	else
		addline("[ ] State", &sort_toggle_state);

	if (screen.Packages->Sorting->Exists(SortOrderList::PkgName))
		addline("[$3#X$!] Pkg Name", &sort_toggle_pkgname);
	else
		addline("[ ] Pkg Name", &sort_toggle_pkgname);

	if (screen.Packages->Sorting->Exists(SortOrderList::Size))
		addline("[$3#X$!] Arch Size", &sort_toggle_size);
	else
		addline("[ ] Arch Size", &sort_toggle_size);

	if (screen.Packages->Sorting->Exists(SortOrderList::Instsize))
		addline("[$3#X$!] Inst Size", &sort_toggle_instsize);
	else
		addline("[ ] Inst Size", &sort_toggle_instsize);

	if (screen.Packages->Sorting->Exists(SortOrderList::Suite))
		addline("[$3#X$!] Suite", &sort_toggle_suite);
	else
		addline("[ ] Suite", &sort_toggle_suite);
}

#ifdef HAVE_LIBGPM
int CursesUI::MenuDrop::gpm_handler(Gpm_Event *event)
{
	if (Gpm_StrictSingle(event->type))
	{
		for (unsigned I = bound_start; I < bound_end; I++)
		{
			unsigned K = I - bound_start;
	
			if (((sy + 2 + K) == event->y) &&
				(sx + 2 <= event->x && event->x <= sx + lx - 1))
			{
				cursor = I;
				drawall();
			}
		}
	}
	else if (Gpm_StrictDouble(event->type))
	{
		for (unsigned I = bound_start; I < bound_end; I++)
		{
			unsigned K = I - bound_start;
	
			if (((sy + 2 + K) == event->y) &&
				(sx + 2 <= event->x && event->x <= sx + lx - 1))
					return -1;
		}
		return -2;
	}
	
	return 0;
}
#endif
