#include <std.h>
#include "deityui.h"

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>

#ifdef USE_DEBCONF
#include <debconf/backend.h>
#include <debconf/itf.h>
#endif

#include <func/messages.h>
#include <func/config.h>
#include "acquireui.h"
#include "pmi.h"

extern "C" {
extern DeityUI *CreateInterface();
};

/*
 * TODO
 * Perhaps using a config file mechanism, load in as dynamic modules
 * different UI interfaces
 */
int DeityUI::LoadUIModules()
{
	return 0;
}

DeityUI *DeityUI::CreateInterface(const char *name, int &argc, char **&argv)
{
#ifdef NODSO
	return ::CreateInterface(argc, argv);
#else
	char buf[256];
	void *handle;
	void *mkfunc;
	DeityUI *ui;

	snprintf(buf, sizeof buf, "%s.so", name);
	handle = dlopen(buf, RTLD_NOW);

	if (handle == NULL)
	{
		fprintf(stderr, "%s\n", dlerror());
		return NULL;
	}
	
	mkfunc = dlsym(handle, "CreateInterface");
	if (mkfunc == NULL)
	{
		fprintf(stderr, "%s: Cannot find UI creator function.\n", buf);
		return NULL;
	}

	ui = ((DeityUI * (*)(int &, char **&))mkfunc) (argc, argv);
	if (ui == NULL)
	{
		fprintf(stderr, "%s: Cannot create UI.\n", buf);
		return NULL;
	}
	return ui;
#endif
}

int DeityUI::Passthrough(int fd)
{
	// Talk to the fetcher directly here
	// Until it is time to return
	return 0;
}

bool DeityUI::ProcessMsg(int fd)
{
	char buf[1024];
	MsgId type;
	unsigned long len = sizeof(buf);
	static vector<MsgDownloadItem> ItemList;
	static MsgDownloadTotal Total;
	static pkgMgrInt *pmi = NULL;

	int ret = MessageReceive(fd, type, buf, len);

	if (ret < 0) return false;

	/* Acquire is modal? 
	   FIXME: I'm only for curses. */
	if (_Conf.GetBool("deity.acquire.embedded") == true)
	{
		switch (type)
		{
			case MsgId_Update_Total:
				ASSERT(len == sizeof(struct MsgDownloadTotal));
				memcpy(&Total, buf, len);
				UpdateAcquireProgress(&Total);
				break;

			case MsgId_Acquire_Start:
				ShowAcquireProgress();
				break;

			case MsgId_Acquire_Stop:
				HideAcquireProgress();
				break;

			default:
				break;
		}
		return true;
	}
					
	switch (type)
	{
		case MsgId_Info: 
		case MsgId_Error: 
			break;
		case MsgId_Acquire_Fatal:
		case MsgId_Acquire_Warning:
		case MsgId_Acquire_Info:
		{
			char txtmsg[len+1];
			strncpy(txtmsg, buf, len);
			txtmsg[len] = 0;
			AcqItf->Message(txtmsg);
			break;
		}		
		case MsgId_MediaChange: 
		{
			ASSERT(len == sizeof(struct MsgMediaChange));
			MsgMediaChange msg;
			memcpy(&msg, buf, len);
			AcqItf->MediaChange(&msg);
			break;
		} 
		case MsgId_Site_Hit: 
		{
			ASSERT(len == sizeof(struct MsgItemDesc));
			MsgItemDesc msg;
			memcpy(&msg, buf, len);
			AcqItf->IMSHit(&msg);
			break;
		}
		case MsgId_Update: 
		{
			ASSERT(len == sizeof(unsigned int));
			unsigned int msg;
			memcpy(&msg, buf, len);
			AcqItf->Update(&ItemList, &Total);
			ItemList.clear();
			break;
		}
		case MsgId_Update_Item: 
		{
			ASSERT(len == sizeof(struct MsgDownloadItem));
			ItemList.resize(ItemList.size() + 1);
			memcpy(&ItemList[ItemList.size() - 1], buf, len);
			break;
		}
		case MsgId_Update_Total: 
		{
			ASSERT(len == sizeof(struct MsgDownloadTotal));
			memcpy(&Total, buf, len);
			break;
		}
		case MsgId_Download_Start: 
		{
			ASSERT(len == sizeof(struct MsgItemDesc));
			MsgItemDesc msg;
			memcpy(&msg, buf, len);
			AcqItf->Fetch(&msg);
			break;
		}
		case MsgId_Download_Complete: 
		{
			ASSERT(len == sizeof(struct MsgItemDesc));
			MsgItemDesc msg;
			memcpy(&msg, buf, len);
			AcqItf->Done(&msg);
			
#ifdef USE_DEBCONF
			if (AcqItf->acqState == AcquireUI::PackageRun)
			{
				printf("preparing to call debconf\n");
				char *DestFile = strrchr(msg.URI, '/');
				if (DestFile != 0) DestFile++;
				DebConfPreconfigure(DestFile);
			}
#endif
			break;
		}
		case MsgId_Download_Failed: 
		{
			ASSERT(len == sizeof(struct MsgItemFailed));
			MsgItemFailed msg;
			memcpy(&msg, buf, len);
			AcqItf->Fail(&msg);
			break;
		}
		case MsgId_Acquire_Start: 
		{
			INFO(INFO_DEBUG, "msg Acquire Start\n");
			AcqItf->Startup();
			AcqItf->Start();
#ifdef USE_DEBCONF
			StartDebConf();
#endif
			break;
		}
		case MsgId_Acquire_Stop: 
		{
			INFO(INFO_DEBUG, "msg Acquire Stop\n");
			AcqItf->Stop();
			AcqItf->Shutdown();
			break;
		}
		case MsgId_Acquire_Shutdown:
		{
			INFO(INFO_DEBUG, "msg Acquire Shutdown\n");
			ASSERT(len == sizeof(bool));
			bool ReinitCache;
			memcpy(&ReinitCache, buf, len);
			signalfd = -1;

			/* Let us theorise, Watson! */
			if (AcqItf != NULL) {
				delete AcqItf;
				AcqItf = NULL;
			}

			/* Also, set the current screen.  SQUISHY BUG! */
			scrlst.SetCurrent(scrlst.Head());

			if (ReinitCache)
				Cache->Reinit();

			/* Finally, redraw the screens */
			CaptUI->DrawScreens();
			break;
		}
		case MsgId_PkgMgr_Start:
		{
			INFO(INFO_DEBUG, "msg Install Start\n");
			bool ready = AcqItf->ReadyToInstall();
			MessageSend(signalfd, MsgId_PkgMgr_Ready, &ready, sizeof(bool));
			pmi = CaptUI->CreatePMI();
			break;
		}	
		case MsgId_PkgMgr_Stop:
		{
			INFO(INFO_DEBUG, "msg Install Stop\n");
#ifdef USE_DEBCONF
			StopDebConf();
#endif
			delete pmi; pmi = NULL;

			/* We send back a MsgId_PkgMgr_Stop so the blocking thread can catch it */
			/* Crucial ---Pat */
			MessageSend(signalfd, MsgId_PkgMgr_Stop, NULL, 0);
			break;
		}
		case MsgId_PkgMgr_Warning:
			INFO(INFO_DEBUG, "msg pkgMgr Warning\n");
			pmi->warning(buf, len);
			break;
		case MsgId_PkgMgr_Output:
			INFO(INFO_DEBUG, "msg pkgMgr Output\n");
			pmi->data(buf, len);
			break;
		default:
			printf("warning: unknown message id == %d", type);
			// NOTREACHED("Unknown message id");
			break;
	}	
	
	return true;
}

#ifdef USE_DEBCONF
bool DeityUI::ProcessDebConfMessage(int fd)
{
	if (DCBackend == 0) return false;

	char dcmsg[1024];
	int ret = read(fd, dcmsg, sizeof(dcmsg));
	if (ret < 0)
	{
		// TODO: Error out
		return false;
	}
	dcmsg[ret] = 0;
	printf("debconf message: [%s]\n", dcmsg);
	DCBackend->Dispatch(dcmsg);
}

void DeityUI::StartDebConf()
{
	CaptUI->CreateDebConfGUI();
	if (DCUI != 0)
	{
		DCBackend = new DebConfBackend(*DCUI);
		WaitForDebConf(DCBackend->GetClientFd());
	}
	printf("DebConf engine started\n");
}

void DeityUI::StopDebConf()
{
	delete DCBackend;
	DCBackend = 0;
	printf("DebConf engine destroyed\n");
}
#endif
