/* $Id*/
#ifndef _dialogs_H
#define _dialogs_H

#include <string>
#include "deityui.h"

// Interface idea for a non-UI-specific selectable, columned list dialog box.
// ...

class DeityUI;

class DeityUI::Dialog
{
protected:
	DeityUI & UI;

public:
	enum DlgFlags { Selectable, Columned };

	struct Column
	{
		int pos;
		void *Heading;
		void *Data;

		Column(int _pos, void *_Heading) : pos(_pos), Heading(_Heading) {};
	};
	typedef unsigned int ColumnIndex;

	struct RowHandler;
	struct Row
	{
		ColumnIndex col;
		void *Data;
		unsigned char key;
		RowHandler (*Handler)();

		Row(ColumnIndex _col, void *_data, unsigned char _key, RowHandler (*)() = NULL) : col(_col), Data(_data), key(_key) {};
	};
	typedef unsigned int RowIndex;

protected:

	DlgFlags Flags;

	vector<Column> ColumnList;
	vector<Row> RowList;

	RowIndex _current, _start, _end;

	//void initialize();    Curses can implement these as only they need to call them
	//void deinitialize(); 

public:

	virtual void Run() = 0;	// select()
	virtual void Shutdown() = 0;	// free any memory used by our lists (they are vectors though..)

	virtual void Draw() = 0;
	virtual void DrawRow(RowIndex) = 0;
	virtual void DrawColumn(ColumnIndex) {};			// Not really necessary, but would be nice :>
	virtual void DrawHeadings() = 0;

	virtual void KeyHandler() {};			// Should -really- implement this

	void set_flag(DlgFlags _flags) { Flags = _flags; };

	virtual ColumnIndex add_column() = 0;
	virtual RowIndex add_row() = 0;

	RowIndex Current() { return _current; };
	RowIndex Start() { return _start; };
	RowIndex End() { return _end; };

public:

	Dialog(DeityUI & _UI) : UI(_UI) {}; 
	virtual ~Dialog();
};

struct DeityUI::Dialog::RowHandler
{
	RowHandler(void (*f) (int));
	RowHandler(void (*f) (void));
	RowHandler(void (DeityUI::*f) (int));
	RowHandler(void (DeityUI::*f) (void));

	enum
	{ 
		NoHandler,
		KeyHandler,
		NullKeyHandler,
		UIHandler,
		NullUIHandler
	} type;

	union
	{
		void (*keyf) (int);
		void (*nullkeyf) (void);
		void (DeityUI::*uif) (int);
		void (DeityUI::*nulluif) (void);
	} fn;

	inline void operator() (int);
};

inline void DeityUI::Dialog::RowHandler::operator() (int key)
{
	if (fn.keyf == 0)
		return;

	switch (type)
	{
	case KeyHandler:
		fn.keyf(key);
		break;
	case NullKeyHandler:
		fn.nullkeyf();
		break;
	case UIHandler:
		(CaptUI->*(fn.uif)) (key);
		break;
	case NullUIHandler:
		(CaptUI->*(fn.nulluif)) ();
		break;
	case NoHandler:
		break;
	}
}
#endif
