/* $Id: filteredit.cc,v 1.2 2001/03/24 08:08:32 ltd Exp $ */
#include <std.h>

#include "filteredit.h"
#include <func/config.h>

static gchar *filterlabels[] = {
	"Name",
	"Available version",
	"Installed version",
	"Section",
	"Priority",
	"Short description",
	"Long description"
};

GtkUI::FilterEditor::FilterEditor(GtkUI &_ui)
	: ui(_ui)
{
	filters = Filter::Target(Filter::pkgname |
		  		 Filter::pkgavailver |
		 		 Filter::pkginstlver |
		 		 Filter::pkgsection |
				 Filter::pkgpriority |
				 Filter::pkgshortdesc |
				 Filter::pkglongdesc);
	baserow = -1;
	targetrow = -1;
	for (int k = 0; k < NUM_FILTER_TYPES; k++)
	{
		baseselected[k] = true;
		targetselected[k] = false;
	}
	dialog = CreateWidget();
}

GtkUI::FilterEditor::~FilterEditor()
{
	gtk_widget_destroy(dialog);
}

GtkWidget *GtkUI::FilterEditor::CreateWidget()
{
	GtkWidget *dlg;
	GtkWidget *butvbox, *hbox;
	GtkWidget *leftbutton, *rightbutton;
	GtkWidget *btnok, *btncancel;

	// Create all the widgets
	dlg = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dlg), "Filtering Options");
	gtk_widget_set_usize(GTK_WIDGET(dlg), _Conf.GetInt("deity.gtk.window.filterselect.xsize"), _Conf.GetInt("deity.gtk.window.filterselect.ysize"));
	hbox = gtk_hbox_new(FALSE, 0);
	butvbox = gtk_vbox_new(FALSE, 0);
	baselist = gtk_clist_new(1);
	targetlist = gtk_clist_new(1);
	leftbutton = gtk_button_new_with_label("<-");
	rightbutton = gtk_button_new_with_label("->");
	filtertext = gtk_entry_new();
	btnok = gtk_button_new_with_label("OK");
	btncancel = gtk_button_new_with_label("Cancel");

	// setup the filter lists
	gtk_clist_set_shadow_type(GTK_CLIST(baselist), GTK_SHADOW_OUT);
	gtk_clist_set_shadow_type(GTK_CLIST(targetlist), GTK_SHADOW_OUT);
	
	// lay them out
	gtk_box_pack_start(GTK_BOX(butvbox), leftbutton, TRUE, FALSE, 10);
	gtk_box_pack_start(GTK_BOX(butvbox), rightbutton, TRUE, FALSE, 10);

	gtk_box_pack_start(GTK_BOX(hbox), baselist, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), butvbox, FALSE, FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox), targetlist, TRUE, TRUE, 0);

	gtk_container_set_border_width(GTK_CONTAINER(dlg), 20);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, TRUE, TRUE, 10);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), gtk_label_new("Text to look for:"), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), filtertext, FALSE, FALSE, 0);

	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->action_area), btnok, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->action_area), btncancel, TRUE, TRUE, 0);

	// Connect the signals
	gtk_signal_connect(GTK_OBJECT(btnok), "clicked",
		GTK_SIGNAL_FUNC(DoOK), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(btncancel), "clicked",
		GTK_SIGNAL_FUNC(DoCancel), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(baselist), "select-row",
		GTK_SIGNAL_FUNC(SelectRow), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(baselist), "unselect-row",
		GTK_SIGNAL_FUNC(UnselectRow), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(targetlist), "select-row",
		GTK_SIGNAL_FUNC(SelectRow), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(targetlist), "unselect-row",
		GTK_SIGNAL_FUNC(UnselectRow), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(leftbutton), "clicked",
		GTK_SIGNAL_FUNC(DoRemove), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(rightbutton), "clicked",
		GTK_SIGNAL_FUNC(DoAdd), (gpointer)this);

	return dlg;

	/*
	gtk_widget_set_sensitive(btnRemove, FALSE);
	*/
}

void GtkUI::FilterEditor::UpdateLists()
{
	gtk_clist_freeze(GTK_CLIST(baselist));
	gtk_clist_freeze(GTK_CLIST(targetlist));
	gtk_clist_clear(GTK_CLIST(baselist));
	gtk_clist_clear(GTK_CLIST(targetlist));
	for (int k = 0; k < NUM_FILTER_TYPES; k++)
	{
		if (baseselected[k])
			gtk_clist_append(GTK_CLIST(baselist), &filterlabels[k]);
		if (targetselected[k])
			gtk_clist_append(GTK_CLIST(targetlist), &filterlabels[k]);
	}
	gtk_clist_thaw(GTK_CLIST(baselist));
	gtk_clist_thaw(GTK_CLIST(targetlist));
}

bool GtkUI::FilterEditor::Run()
{
	UpdateLists();
	gtk_widget_show_all(dialog);
	gtk_main();
	INFO(INFO_DEBUG, "Closing filter editor window; cancelled = %s\n",
		cancelled ? "true" : "false");
	return !cancelled;
}

void GtkUI::FilterEditor::DoOK(GtkWidget *, gpointer data)
{
	GtkUI::FilterEditor *This = (GtkUI::FilterEditor *)data;
	gtk_main_quit();
	This->cancelled = false;
}

void GtkUI::FilterEditor::DoCancel(GtkWidget *, gpointer data)
{
	GtkUI::FilterEditor *This = (GtkUI::FilterEditor *)data;
	gtk_main_quit();
	This->cancelled = true;
}

void GtkUI::FilterEditor::GetFilter(char *&txt, Filter::Target &target)
{
	int targ = 0;
	gchar *tmp = gtk_entry_get_text(GTK_ENTRY(filtertext));
	if (tmp == NULL)
		txt = strdup("");
	else
		txt = strdup(tmp);

	if (targetselected[0])
		targ |= Filter::pkgname;
	if (targetselected[1])
		targ |= Filter::pkgavailver;
	if (targetselected[2])
		targ |= Filter::pkginstlver;
	if (targetselected[3])
		targ |= Filter::pkgsection;
	if (targetselected[4])
		targ |= Filter::pkgpriority;
	if (targetselected[5])
		targ |= Filter::pkgshortdesc;
	if (targetselected[6])
		targ |= Filter::pkglongdesc;

	target = Filter::Target(targ);
}

void GtkUI::FilterEditor::SelectRow(GtkCList *clist, gint row, gint col,
	GdkEventButton *event, gpointer data)
{
	GtkUI::FilterEditor *This = (GtkUI::FilterEditor *)data;
	gchar *text = 0;
	int r = 0;

	gtk_clist_get_text(clist, row, col, &text);
	INFO(INFO_DEBUG, "selected text is \"%s\"\n", text);
	for (; r < NUM_FILTER_TYPES; r++)
		if (strcmp(text, filterlabels[r]) == 0)
			break;

	if (r == NUM_FILTER_TYPES)
		return;

	if (clist == GTK_CLIST(This->baselist))
		This->baserow = r;
	else if (clist == GTK_CLIST(This->targetlist))
		This->targetrow = r;
}

void GtkUI::FilterEditor::UnselectRow(GtkCList *clist, gint row, gint col,
	GdkEventButton *event, gpointer data)
{
	GtkUI::FilterEditor *This = (GtkUI::FilterEditor *)data;
	if (clist == GTK_CLIST(This->baselist))
		This->baserow = -1;
	else if (clist == GTK_CLIST(This->targetlist))
		This->targetrow = -1;
}

void GtkUI::FilterEditor::DoAdd(GtkWidget *, gpointer data)
{
	GtkUI::FilterEditor *This = (GtkUI::FilterEditor *)data;
	if (This->baserow < 0) return;
	This->baseselected[This->baserow] = false;
	This->targetselected[This->baserow] = true;
	This->UpdateLists();
}

void GtkUI::FilterEditor::DoRemove(GtkWidget *, gpointer data)
{
	GtkUI::FilterEditor *This = (GtkUI::FilterEditor *)data;
	if (This->targetrow < 0) return;
	This->baseselected[This->targetrow] = true;
	This->targetselected[This->targetrow] = false;
	This->UpdateLists();
}

