/* $Id: pkgtreewidget.cc,v 1.13 2001/05/15 10:11:07 ltd Exp $ */
#include <std.h>
#include <stdio.h>
#include <func/apt.h>
#include <func/config.h>
#include <func/pkgchecks.h>
#include <func/tree.h>
#include <interface/format_strings.h>
#include <list/sections.h>
#include "screen.h"
#include "pkgtreewidget.h"
#include "util.h"
#include "pixmaps/package.xpm"
#include "pixmaps/divider.xpm"
#include "pixmaps/depends.xpm"
#include "pixmaps/conflicts.xpm"
#include "pixmaps/replaces.xpm"
#include "pixmaps/recommends.xpm"
#include "pixmaps/suggests.xpm"
#include "pixmaps/tree.xpm"
#include "pixmaps/tree_minus.xpm"
#include "pixmaps/tree_plus.xpm"
#include "pixmaps/tree_minus_term.xpm"
#include "pixmaps/tree_plus_term.xpm"
#include "pixmaps/tree_term.xpm"

#define HOVERINTERVAL	2000   // ms

#define GTKUI_PKG_ICON_SIZE 16
#define GTKUI_TREE_ICON_SIZE 20
#define GTKUI_PKG_INDENT 11 

#define GETGC(gc, state) 				\
	switch (state) {				\
		case Normal: gc = normal_gc; break;	\
		case Selected: gc = selected_gc; break;	\
		case Prelight: gc = prelight_gc; break;	\
		case Matched: gc = matched_gc; break;	\
		default: /* bad!!! */ NOTREACHED("Unknown state"); break;	\
	}

using namespace GtkUtil;

gchar *GtkUI::PkgTreeWidget::titles[] = { "Package", "Is", "Will", "Avl. Vers", "Ins. Vers", "Description" };

GdkPixmap *GtkUI::PkgTreeWidget::package_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::divider_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::depends_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::conflicts_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::replaces_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::recommends_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::suggests_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::expanded_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::contracted_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::expanded_term_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::contracted_term_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::tree_nochild_term_pixmap[4];
GdkPixmap *GtkUI::PkgTreeWidget::tree_nochild_pixmap[4];
#define GETPIXMAP(pixmap, state) (state == Normal ? pixmap[0] : (state == Selected ? pixmap[1] : (state == Prelight ? pixmap[2] : pixmap[3])))

GtkUI::PkgTreeWidget::PkgTreeWidget(GtkUI::Screen & _screen)
	: screen(_screen), normal_gc(0), prelight_gc(0), selected_gc(0), matched_gc(0), bpixmap(0), adjustment(0), vspacing(20), redrawqueued(false), resizingcolumn(-1)
{
	/* these can't be created in the static constructor because they need 
	 * a window context. kinda stupid.... */

	GdkColor *color, cursor, prelight, normal, matched;

	normal.red     = 0xffff;
	normal.green   = 0xffff;
	normal.blue    = 0xffff;
	cursor.red     = 0;
	cursor.green   = 0;
	cursor.blue    = 0;
	prelight.red   = 0xe500;
	prelight.green = 0xe500;
	prelight.blue  = 0xe500;
	matched.red   = 0xff00;
	matched.green = 0xdd00;
	matched.blue  = 0xff00;

	for (int i = 0; i < 4; i++)
	{
		if (i == 0) color = &normal;
		else if (i == 1) color = &cursor;
		else if (i == 2) color = &prelight;
		else color = &matched;

		package_pixmap[i]    = PixmapFromXPM((char **) package_xpm, color);
		divider_pixmap[i]    = PixmapFromXPM((char **) divider_xpm, color);
		depends_pixmap[i]    = PixmapFromXPM((char **) depends_xpm, color);
		conflicts_pixmap[i]  = PixmapFromXPM((char **) conflicts_xpm, color);
		replaces_pixmap[i]   = PixmapFromXPM((char **) replaces_xpm, color);
		recommends_pixmap[i] = PixmapFromXPM((char **) recommends_xpm, color);
		suggests_pixmap[i]   = PixmapFromXPM((char **) suggests_xpm, color);
		expanded_pixmap[i]   = PixmapFromXPM((char **) tree_minus_xpm, color);
		contracted_pixmap[i] = PixmapFromXPM((char **) tree_plus_xpm, color);
		expanded_term_pixmap[i]   = PixmapFromXPM((char **) tree_minus_term_xpm, color);
		contracted_term_pixmap[i] = PixmapFromXPM((char **) tree_plus_term_xpm, color);
		tree_nochild_pixmap[i] = PixmapFromXPM((char **) tree_xpm, color);
		tree_nochild_term_pixmap[i] = PixmapFromXPM((char **) tree_term_xpm, color);
	}

	/* Set up a font for the tree */
	if ((treefont = gdk_font_load(_Conf.Get("deity.gtk.font.packages"))) == NULL) {
		fprintf(stderr, "Cannot load pkgtree font: %s, trying fixed..\n", _Conf.Get("deity.gtk.font.packages"));
		if ((treefont = gdk_font_load("fixed")) == NULL) {
			fprintf(stderr, "Unable to load font fixed, exiting..\n");
			exit(0);
		}
	}
	vspacing = 20;		// should not be hardcoded?

	// This gets overridden in the resize event below
	memset(columnoffsets, 0, sizeof(columnoffsets));

	SetupGCs();

	/* Create the owner-draw tree widget */
	theTree = gtk_drawing_area_new();

	GTK_WIDGET_SET_FLAGS(theTree, GTK_CAN_FOCUS);

	// Capture the events we want
	gtk_widget_set_events(theTree, GDK_EXPOSURE_MASK
			      | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
			      GDK_KEY_PRESS_MASK | GDK_POINTER_MOTION_MASK |
			      GDK_POINTER_MOTION_HINT_MASK | GDK_LEAVE_NOTIFY_MASK);

	gtk_signal_connect(GTK_OBJECT(theTree), "realize", GTK_SIGNAL_FUNC(Realize), this);
	gtk_signal_connect(GTK_OBJECT(theTree), "configure-event", GTK_SIGNAL_FUNC(Resize),
			   this);
	gtk_signal_connect(GTK_OBJECT(theTree), "expose-event", GTK_SIGNAL_FUNC(Expose), this);
	gtk_signal_connect(GTK_OBJECT(theTree), "button_press_event",
			   GTK_SIGNAL_FUNC(ButtonPress), this);
	gtk_signal_connect(GTK_OBJECT(theTree), "button_release_event",
			   GTK_SIGNAL_FUNC(ButtonRelease), this);
	gtk_signal_connect(GTK_OBJECT(theTree), "motion_notify_event",
			   GTK_SIGNAL_FUNC(MotionNotify), this);
	gtk_signal_connect(GTK_OBJECT(theTree), "leave_notify_event",
			   GTK_SIGNAL_FUNC(LeaveNotify), this);

}

GtkUI::PkgTreeWidget::~PkgTreeWidget()
{
	/* Save the values of the column offsets before we die */
	char buf[10];
	snprintf(buf, 10, "%f", (float) realcoloffsets[1] / theTree->allocation.width);
	_Conf.Set("deity.gtk.tree.columnoffset.1", buf);
	snprintf(buf, 10, "%f", (float) realcoloffsets[2] / theTree->allocation.width);
	_Conf.Set("deity.gtk.tree.columnoffset.2", buf);
	snprintf(buf, 10, "%f", (float) realcoloffsets[3] / theTree->allocation.width);
	_Conf.Set("deity.gtk.tree.columnoffset.3", buf);
	snprintf(buf, 10, "%f", (float) realcoloffsets[4] / theTree->allocation.width);
	_Conf.Set("deity.gtk.tree.columnoffset.4", buf);
	snprintf(buf, 10, "%f", (float) realcoloffsets[5] / theTree->allocation.width);
	_Conf.Set("deity.gtk.tree.columnoffset.5", buf);
}

/* static event handlers */
gint GtkUI::PkgTreeWidget::Resize(GtkWidget * widget, GdkEventConfigure * event, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;

	if (tree->bpixmap != 0)
	{
		gdk_pixmap_unref(tree->bpixmap);
	}

	tree->bpixmap = gdk_pixmap_new(widget->window, widget->allocation.width,
		widget->allocation.height, -1);

	/* these are cumulative offsets */

	tree->columnoffsets[0] = (0 * widget->allocation.width);
	tree->realcoloffsets[0] = (0 * widget->allocation.width);

	tree->columnoffsets[1] = (int)(_Conf.GetFloat("deity.gtk.tree.columnoffset.1") * widget->allocation.width);
	tree->realcoloffsets[1] = (_Conf.GetFloat("deity.gtk.tree.columnoffset.1") * widget->allocation.width);

	tree->columnoffsets[2] = (int)(_Conf.GetFloat("deity.gtk.tree.columnoffset.2") * widget->allocation.width);
	tree->realcoloffsets[2] = (_Conf.GetFloat("deity.gtk.tree.columnoffset.2") * widget->allocation.width);
	
	tree->columnoffsets[3] = (int)(_Conf.GetFloat("deity.gtk.tree.columnoffset.3") * widget->allocation.width);
	tree->realcoloffsets[3] = (_Conf.GetFloat("deity.gtk.tree.columnoffset.3") * widget->allocation.width);

	tree->columnoffsets[4] = (int)(_Conf.GetFloat("deity.gtk.tree.columnoffset.4") * widget->allocation.width);
	tree->realcoloffsets[4] = (_Conf.GetFloat("deity.gtk.tree.columnoffset.4") * widget->allocation.width);

	tree->columnoffsets[5] = (int)(_Conf.GetFloat("deity.gtk.tree.columnoffset.5") * widget->allocation.width);
	tree->realcoloffsets[5] = (_Conf.GetFloat("deity.gtk.tree.columnoffset.5") * widget->allocation.width);

	tree->columnoffsets[6] = widget->allocation.width;
	tree->realcoloffsets[6] = widget->allocation.width;

	tree->screen.BoundsReset();
	tree->UpdateAdjustment();
	tree->QueueRedraw();

	return TRUE;
}

gint GtkUI::PkgTreeWidget::Expose(GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;

	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
			tree->bpixmap, event->area.x, event->area.y, event->area.x,
			event->area.y, event->area.width, event->area.height);

	return FALSE;
}

void GtkUI::PkgTreeWidget::Realize(GtkWidget * widget, gpointer data)
{
	//GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	// what should we really do here?
	gdk_window_set_background(widget->window, &widget->style->base[GTK_STATE_NORMAL]);
}

// tmp debug func
void GtkUI::PkgTreeWidget::PrintPackage(::PackageTree::Item * p)
{
	if (p->IsDivider())
	{
		string format = screen.Packages->GetHeaderDesc(p);
		g_print("%s", format.c_str());
	}
	else
	{
		g_print("%s", p->TargetPkg.Name());
	}
}

gboolean GtkUI::PkgTreeWidget::ButtonPress(GtkWidget * w, GdkEventButton * event, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	int oldcursor = tree->screen._cursor - tree->screen._boundstart;
	int newcursor = (int) (event->y / tree->vspacing) - 1;

	/*
	   g_print("Button press event: x=%.00f,y=%.00f\n", event->x, event->y);
	   g_print("cursor = %d, boundstart = %d\n", tree->screen._cursor, tree->screen._boundstart);
	   g_print("oldcur = %d, newcur = %d\n", oldcursor, newcursor);
	   g_print("Old cursor is on package: ");
	   tree->PrintPackage(tree->screen.cs);
	   g_print("\n");
	*/

	gdk_pointer_grab(tree->theTree->window, FALSE,
			 (GdkEventMask) (GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK),
			 NULL, NULL, event->time);
	if (newcursor < 0 || newcursor > (tree->screen._boundend - tree->screen._boundstart))
		return 0;	// outside of bounds 

	switch (event->type)
	{
	case GDK_BUTTON_PRESS:
		/* highlight the package */
		if (event->button == 1)
		{
			tree->MoveCursor(newcursor - oldcursor);
			/* left mouse click-- 
			   check to see if the user clicked in an expand/collapse box

			if (fabs(tree->screen.cs->increment * GTKUI_PKG_INDENT +
				 GTKUI_PKG_ICON_SIZE / 2 - event->x) < 5
			    && fabs((newcursor + 1) * tree->vspacing + 5 +
				    GTKUI_PKG_ICON_SIZE / 2 - event->y) < 5)
				g_print("Inside the square!\n");
			*/

			tree->screen.UpdateDescription();
		}
		else if (event->button == 3)
		{		/* right mouse click */
			tree->MoveCursor(newcursor - oldcursor);
			tree->screen.UpdateDescription();

			switch (tree->screen.cs->Type)
			{
			case ::PackageTree::Item::Div_State:
			case ::PackageTree::Item::Div_Sect:
			case ::PackageTree::Item::Div_Pri:
			case ::PackageTree::Item::Div_Custom:
				break;
			default:
				GtkWidget * menu = tree->MakeContextMenu(tree->screen.cs);
				gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
					       event->button, event->time);
				return TRUE;
			}
		} 
		else if (event->button == 4)
		{
			tree->screen.ArrowUp();
		}
		else if (event->button == 5)
		{
			tree->screen.ArrowDown();
		}
		return TRUE;

	case GDK_2BUTTON_PRESS:
		/* expand the package (?) */
		if (event->button == 1)
			tree->DependToggle();
		return TRUE;
	default:
		/* nothing */ ;
	};

	return FALSE;
}

gboolean GtkUI::PkgTreeWidget::ButtonRelease(GtkWidget * w, GdkEventButton * event, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	//g_print("Button release event: x=%.00f,y=%.00f\n", event->x, event->y);
	gdk_pointer_ungrab(event->time);
	if (tree->resizingcolumn > 0) {
		tree->resizingcolumn = -1;
		gdk_window_set_cursor(tree->theTree->window, NULL);
		tree->QueueRedraw();
	}
	return 0;
}

gboolean GtkUI::PkgTreeWidget::MotionNotify(GtkWidget * w, GdkEventMotion * event, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	gint x, y;
	GdkModifierType state;

	if (event->is_hint)
	{
		gdk_window_get_pointer(event->window, &x, &y, &state);
	}
	else
	{
		x = (gint) event->x;
		y = (gint) event->y;
		state = (GdkModifierType) event->state;
	}

	int cursor = y / tree->vspacing - 1;
	if (cursor > (tree->screen._boundend - tree->screen._boundstart))
	{
		/* outside of bounds */
		tree->RemovePrelight();
		return 0;
	}
		
	if (cursor < 0)
	{
		// in heading row, check for column resizing
		if (state & GDK_BUTTON1_MASK)
		{
			//g_print("x = %d, y = %d, b1 = %s\n", x, y, (state & GDK_BUTTON1_MASK ? "on" : "off"));
			if (tree->resizingcolumn < 0) 
				for (int col = 1; col < GTKUI_PKGTREE_COLUMNS; col++) {
				//g_print("x = %d, column[%d] = %d\n", x, col, tree->columnoffsets[col]);
					// give a bit of room
					if (abs(x - tree->columnoffsets[col]) < 3) {
						tree->resizingcolumn = col;
						GdkCursor *cursor = gdk_cursor_new(GDK_SB_H_DOUBLE_ARROW);
						gdk_window_set_cursor(tree->theTree->window, cursor);
						gdk_cursor_destroy(cursor);
						break;
					}
				}
			tree->ColumnResize(x);
		}
		return 0;
	}
	if (tree->resizingcolumn > 0)
		tree->ColumnResize(x);

	/*
	 * --That whole MoveCursor thing is eeeevil.
	 * <captain>  The borg have adapted!   </captain>
	 */

	if (cursor != tree->screen._prelight)
		tree->DoPrelight(cursor);
	return 0;
}

gboolean GtkUI::PkgTreeWidget::LeaveNotify(GtkWidget * w, GdkEventCrossing * event, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	tree->RemovePrelight();
	return 0;
}
	
void GtkUI::PkgTreeWidget::ColumnResize(gint x)
{
	if (resizingcolumn <= 0) return;

	int offset = x - columnoffsets[resizingcolumn];
	
	// check if we are resizing past the previous or next column
	if ((offset > 0 && x > columnoffsets[resizingcolumn+1]) ||
	    (offset < 0 && x < columnoffsets[resizingcolumn-1]))
		return;
		
	for (int j = resizingcolumn; j < GTKUI_PKGTREE_COLUMNS; j++)
	{
		columnoffsets[j] += offset;
		realcoloffsets[j] += offset;
	}
	QueueRedraw();
}

void GtkUI::PkgTreeWidget::MarkKeep(GtkMenuItem * item, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	pkgCache::PkgIterator p = tree->screen.cs->TargetPkg;

	(*Cache)->MarkKeep(p);
	(*Cache)->SetReInstall(p, false);

	/* Should only redraw this package */
	tree->QueueRedraw();
}

void GtkUI::PkgTreeWidget::MarkInstall(GtkMenuItem * item, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	if (tree->screen.cs->Type != ::PackageTree::Item::Pkg)
		return;

	pkgCache::PkgIterator p = tree->screen.cs->TargetPkg;

	TryInstall(p);

	/* TODO: Because installing one package can trigger others to change state. 
	   However, perhaps an algorithm to check the deps of the package and then
		 look for them on the screen and redraw those items which have changed.
		 This may be wise considering redraws are slow in gtk -- Pat */
	tree->QueueRedraw();
}

void GtkUI::PkgTreeWidget::MarkUpgrade(GtkMenuItem * item, gpointer data)
{
	MarkInstall(item, data);
}

void GtkUI::PkgTreeWidget::MarkReinstall(GtkMenuItem * item, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	if (tree->screen.cs->Type != ::PackageTree::Item::Pkg)
		return;

	pkgCache::PkgIterator p = tree->screen.cs->TargetPkg;

	TryInstall(p, true);

	/* Should only redraw this package */
	tree->QueueRedraw();
}

void GtkUI::PkgTreeWidget::MarkRemove(GtkMenuItem * item, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	pkgCache::PkgIterator p = tree->screen.cs->TargetPkg;

	TryRemove(p);

	// probably should only redraw this one item 
	tree->QueueRedraw();
}

void GtkUI::PkgTreeWidget::MarkPurge(GtkMenuItem * item, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	pkgCache::PkgIterator p = tree->screen.cs->TargetPkg;

	TryRemove(p, true);

	// probably should only redraw this one item 
	tree->QueueRedraw();
}

void GtkUI::PkgTreeWidget::MarkHold(GtkMenuItem * item, gpointer data)
{
	GtkUI::PkgTreeWidget * tree = (GtkUI::PkgTreeWidget *) data;
	if (tree->screen.cs->Type != ::PackageTree::Item::Pkg)
		return;

	// we don't do this yet (?)
}


/* ----------------------------- 
 * End of event handlers 
 * ----------------------------- */

int GtkUI::PkgTreeWidget::DoPrelight(int y_ref)
{
	::PackageTree::Iterator T;
	int nc;
	GdkRectangle update_rect;

	// Change state of old prelitten object
	if (!screen.pl.Invalid())
	{
		Drawline(screen.pl, screen._prelight, false);
		update_rect.x = 0;
		update_rect.y = (screen._prelight + 1) * vspacing;
		update_rect.width = theTree->allocation.width;
		update_rect.height = vspacing;
		gtk_widget_draw(theTree, &update_rect);
	}

	if (y_ref > screen.Packages->Length())
	{
		screen.pl = (::PackageTree::Item *) 0;
		return 0;
	}

	// Figure out place of the new one
	for (T = screen.bs, nc = 0; !T.Invalid() && nc < y_ref; nc++, T++);
	// g_print("Actually doing prelight: y_ref=%d pkg=%s bs=%d, bspkg=%s\n", y_ref, P->TargetPkg.Name(), screen._boundstart, screen.boundstart->TargetPkg.Name());

	// Light it if we should be doing that
	if (T.Invalid()) return 0;
	Drawline(T, y_ref, y_ref + screen._boundstart == screen._cursor ? false : true);
	update_rect.x = 0;
	update_rect.y = (y_ref + 1) * vspacing;
	update_rect.width = theTree->allocation.width;
	update_rect.height = vspacing;
	gtk_widget_draw(theTree, &update_rect);

	/* Set up the prelight variables for next round baybe */
	screen._prelight = y_ref;
	screen.pl = T;

	return 0;
}

void GtkUI::PkgTreeWidget::RemovePrelight()
{
	if (screen.pl.Invalid())
		return;

	// Change state of old prelitten object
	Drawline(screen.pl, screen._prelight, false);

	/* tell gtk to update the two lines we updated */
	GdkRectangle update_rect;
	update_rect.x = 0;
	update_rect.y = (screen._prelight + 1) * vspacing;
	update_rect.width = theTree->allocation.width;
	update_rect.height = vspacing;
	gtk_widget_draw(theTree, &update_rect);

	screen.pl = (::PackageTree::Item *) 0;
	screen._prelight = -1;
}

/* Set up the GCs */
void GtkUI::PkgTreeWidget::SetupGCs()
{
	GdkColor white, black, gray, magenta;

	if (normal_gc != 0)
		return;		// only do this once!

	normal_gc = gdk_gc_new(screen.window->window);
	selected_gc = gdk_gc_new(screen.window->window);
	prelight_gc = gdk_gc_new(screen.window->window);
	matched_gc = gdk_gc_new(screen.window->window);

	GdkColormap *cmap = gdk_colormap_get_system();
	gdk_color_white(cmap, &white);
	gdk_color_black(cmap, &black);

	gray.red = gray.green = gray.blue = 0xe500;
	gdk_color_alloc(cmap, &gray);
	magenta.red = 0xff00; magenta.green = 0xdd00; magenta.blue = 0xff00;
	gdk_color_alloc(cmap, &magenta);

	gdk_gc_set_foreground(normal_gc, &black);
	gdk_gc_set_background(normal_gc, &white);

	gdk_gc_set_foreground(selected_gc, &white);
	gdk_gc_set_background(selected_gc, &black);

	gdk_gc_set_foreground(prelight_gc, &black);
	gdk_gc_set_background(prelight_gc, &gray);

	gdk_gc_set_foreground(matched_gc, &black);
	gdk_gc_set_background(matched_gc, &magenta);
}

void GtkUI::PkgTreeWidget::ScrollEvent(GtkAdjustment * adj)
{
	while (adj->value > screen._cursor)
	{
		screen.cs++;
		screen._cursor++;
	}
	while (adj->value < screen._cursor)
	{
		screen.cs--;
		screen._cursor--;
	}
	screen._prelight = screen._cursor;
	screen.pl = screen.cs;

	while (screen._boundend < screen._cursor)
	{
		screen.bs++;
		screen._boundstart++;
		screen.be++;
		screen._boundend++;
	}

	while (screen._boundstart > screen._cursor)
	{
		screen.bs--;
		screen._boundstart--;
		screen.be--;
		screen._boundend--;
	}

	QueueRedraw();
}

void GtkUI::PkgTreeWidget::UpdateAdjustment(void)
{
	if (adjustment == 0)
		return;

	int count = screen.Packages->Length();
	int pagesize = LinesInView();
	if (pagesize > count) pagesize = count;

	adjustment->lower = 0;	// this never really changes
	adjustment->upper = count + pagesize - 1;
	adjustment->step_increment = 1;
	adjustment->page_increment = pagesize;
	adjustment->page_size = pagesize;

	//g_print("Adjustments updated; new upperbound is %d\n", count - 1);
	gtk_adjustment_changed(adjustment);
}

int GtkUI::PkgTreeWidget::LinesInView()
{
	return theTree->allocation.height / vspacing - 1;
}

void GtkUI::PkgTreeWidget::DrawHeading()
{
	// Draws the column headings with a nice 3-d look
	GdkColor gray, black, white;
	GdkGC *gc;
	int height = vspacing - 1;
	int width = theTree->allocation.width - 1;
	GdkColormap *cmap = gdk_colormap_get_system();
	gdk_color_black(cmap, &black);
	gdk_color_white(cmap, &white);

	gray.red = gray.green = gray.blue = 0xDDDD;
	gdk_color_alloc(cmap, &gray);

	gc = gdk_gc_new(screen.window->window);
	gdk_gc_set_line_attributes(gc, 2, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_BEVEL);
	gdk_gc_set_foreground(gc, &gray);
	gdk_gc_set_background(gc, &white);

	gdk_draw_rectangle(bpixmap, gc, TRUE, 0, 0, width, height);

	int i = 0;
	for (; i < GTKUI_PKGTREE_COLUMNS; i++)
	{
		gdk_gc_set_foreground(gc, &white);
		/* left */
		gdk_draw_line(bpixmap, gc, columnoffsets[i], 0, columnoffsets[i], height);
		/* top */
		gdk_draw_line(bpixmap, gc, columnoffsets[i], 0, columnoffsets[i + 1], 0);

		gdk_gc_set_foreground(gc, &black);
		/* right */
		gdk_draw_line(bpixmap, gc, columnoffsets[i + 1] - 1, 0,
			      columnoffsets[i + 1] - 1, height);
		/* bottom */
		gdk_draw_line(bpixmap, gc, columnoffsets[i], height, columnoffsets[i + 1],
			      height);

		gdk_draw_string(bpixmap, treefont, gc, columnoffsets[i] + 3, height - 6,
				titles[i]);
	}

}

void GtkUI::PkgTreeWidget::BlankLine(SelectState state, gint ypos)
{
	/* this seems terribly inefficient :-( I don't understand why the fill 
	 * function fills with the *foreground* color and not the *background* 
	 */
	GdkGC *gc = NULL;
	GdkGC *reversegc = gdk_gc_new(screen.window->window);
	GdkGCValues values;

	GETGC(gc, state);
	gdk_gc_copy(reversegc, gc);
	gdk_gc_get_values(reversegc, &values);
	gdk_gc_set_foreground(reversegc, &values.background);
	gdk_gc_set_background(reversegc, &values.foreground);

	gdk_draw_rectangle(bpixmap, reversegc, TRUE, 0, (ypos + 1) * vspacing,
			   theTree->allocation.width, vspacing);

	gdk_gc_unref(reversegc);
}

void GtkUI::PkgTreeWidget::DrawHeader(::PackageTree::Item * p, SelectState state, gint ypos)
{
	GdkGC *gc = NULL;
	GdkPixmap *ec_pixmap = NULL;
	int Inc = p->increment;

	BlankLine(state, ypos);
	GETGC(gc, state);

	if (p->child) {
		if (p->IsExpanded()) {
			if (p->next)
				ec_pixmap = GETPIXMAP(expanded_pixmap, state);
			else
				ec_pixmap = GETPIXMAP(expanded_term_pixmap, state);
		}
		else {
			if (p->next)
				ec_pixmap = GETPIXMAP(contracted_pixmap, state);
			else
				ec_pixmap = GETPIXMAP(contracted_term_pixmap, state);
		}
	} else {
		if (p->next)
			ec_pixmap = GETPIXMAP(tree_nochild_pixmap, state);
		else
			ec_pixmap = GETPIXMAP(tree_nochild_term_pixmap, state);
	}
		

	string format = screen.Packages->GetHeaderDesc(p);

	gdk_draw_pixmap(bpixmap, gc, ec_pixmap, 0, 0,
			(Inc * GTKUI_PKG_INDENT) + 5,
			(ypos + 1) * vspacing, 
			GTKUI_TREE_ICON_SIZE, GTKUI_TREE_ICON_SIZE);

	gdk_draw_pixmap(bpixmap, gc, GETPIXMAP(divider_pixmap, state), 0, 0,
			(Inc * GTKUI_PKG_INDENT) + 5 + GTKUI_TREE_ICON_SIZE + 5,
			(ypos + 1) * vspacing + 2, 
			GTKUI_PKG_ICON_SIZE, GTKUI_PKG_ICON_SIZE);

	gdk_draw_string(bpixmap, treefont, gc,
			(Inc * GTKUI_PKG_INDENT) + 5 + GTKUI_TREE_ICON_SIZE + 5 + GTKUI_PKG_ICON_SIZE + 3,
			(ypos + 2) * vspacing - 6, format.c_str());

	PackageTree::Item *q = p;
	int X = Inc * GTKUI_PKG_INDENT;
	while (q->parent && X > 0)
	{
		X -= GTKUI_PKG_INDENT;

		if (q->parent->next)
			gdk_draw_line(bpixmap, gc,
				/* x1, y1 */ X + 7, (ypos + 1) * vspacing,
				/* x2, y2 */ X + 7, ((ypos + 2) * vspacing) - 1);

		q = q->parent;
	}
}

void GtkUI::PkgTreeWidget::DrawPackage(::PackageTree::Item * p, SelectState state, gint ypos)
{
	GdkGC *gc = NULL;
	GdkPixmap *ec_pixmap = NULL;

	BlankLine(state, ypos);
	GETGC(gc, state);

	if (p->Version().end() == false) {
		if (p->Version().DependsList().end() == false) {
			if (p->IsExpanded()) {
				if (p->next)
					ec_pixmap = GETPIXMAP(expanded_pixmap, state);
				else
					ec_pixmap = GETPIXMAP(expanded_term_pixmap, state);
			}
			else {
				if (p->next)
					ec_pixmap = GETPIXMAP(contracted_pixmap, state);
				else
					ec_pixmap = GETPIXMAP(contracted_term_pixmap, state);
			}
		} else {
			if (p->next)
				ec_pixmap = GETPIXMAP(tree_nochild_pixmap, state);
			else
				ec_pixmap = GETPIXMAP(tree_nochild_term_pixmap, state);
		}
	} else {
		if (p->next)
			ec_pixmap = GETPIXMAP(tree_nochild_pixmap, state);
		else
			ec_pixmap = GETPIXMAP(tree_nochild_term_pixmap, state);
	}

	gdk_draw_pixmap(bpixmap, gc, ec_pixmap, 0, 0,
			((p->increment) * GTKUI_PKG_INDENT) + 5,
			(ypos + 1) * vspacing, 
			GTKUI_TREE_ICON_SIZE, GTKUI_TREE_ICON_SIZE);

	gdk_draw_pixmap(bpixmap, gc, GETPIXMAP(package_pixmap, state), 0, 0,
			((p->increment) * GTKUI_PKG_INDENT) + 5 + GTKUI_TREE_ICON_SIZE + 5,
			(ypos + 1) * vspacing + 2, 
			GTKUI_PKG_ICON_SIZE, GTKUI_PKG_ICON_SIZE);

	gdk_draw_string(bpixmap, treefont, gc,
			((p->increment) * GTKUI_PKG_INDENT) + 5 + GTKUI_TREE_ICON_SIZE + 5 + GTKUI_PKG_ICON_SIZE + 5,
			(ypos + 2) * vspacing - 6, 
			p->TargetPkg.Name());

	pkgCache::VerIterator V, InstV;
	pkgCache::PkgIterator I = p->TargetPkg;
	V = (*Cache)[I].CandidateVerIter(*Cache);
	InstV = (*Cache)[I].InstVerIter(*Cache);

	gdk_draw_string(bpixmap, treefont, gc, columnoffsets[1] + 3, (ypos + 2) * vspacing - 6, (*Cache)[I].Status == 1 ? "Installed" : ((InstV.end() == false || (*Cache)[I].Delete() == true) && (*Cache)[I].NewInstall() == false ? "Upgraded" : "Kept")); 

	gdk_draw_string(bpixmap, treefont, gc, columnoffsets[2] + 3, (ypos + 2) * vspacing - 6, ((*Cache)[I].InstBroken()? "Broken" : ((*Cache)[I].  NewInstall()? "Install" : ((*Cache)[I].  Upgrade()?  "Upgrade" : ((*Cache)[I].  Downgrade()?  "Delete" : ((*Cache) [I].  Install()?  "Install" : ( (*Cache) [I].  Delete ()? ( ( (*Cache) [I].  iFlags & pkgDepCache:: Purge) ? "Purge" : "Remove") : ((*Cache)[I].iFlags & pkgDepCache::ReInstall) ? "Reinstall" : "Keep")))))));

	if (V.end() == false)
		gdk_draw_string(bpixmap, treefont, gc, columnoffsets[3] + 3, (ypos + 2) * vspacing - 6, V.VerStr());

	if (InstV.end() == false)
		gdk_draw_string(bpixmap, treefont, gc, columnoffsets[4] + 3, (ypos + 2) * vspacing - 6, InstV.VerStr());

	if (V.end() == false)
		gdk_draw_string(bpixmap, treefont, gc, columnoffsets[5] + 3, (ypos + 2) * vspacing - 6, Cache->GetPkgRecs().Lookup(V.FileList()).ShortDesc().c_str());

	PackageTree::Item *q = p;
	int X = p->increment * GTKUI_PKG_INDENT;
	while (q->parent && X > 0)
	{
		X -= GTKUI_PKG_INDENT;

		if (q->parent->next)
			gdk_draw_line(bpixmap, gc,
				/* x1, y1 */ X + 7, (ypos + 1) * vspacing,
				/* x2, y2 */ X + 7, ((ypos + 2) * vspacing) - 1);

		q = q->parent;
	}
}

void GtkUI::PkgTreeWidget::DrawDepend(::PackageTree::Item * p, SelectState state, gint ypos)
{
	GdkGC *gc = NULL;
	GdkPixmap *ec_pixmap = NULL;
	GdkPixmap *icon = GETPIXMAP(package_pixmap, state);

	BlankLine(state, ypos);
	GETGC(gc, state);

	if (p->Version().end() == false) {
		if (p->Version().DependsList().end() == false) {
			if (p->IsExpanded()) {
				if (p->next)
					ec_pixmap = GETPIXMAP(expanded_pixmap, state);
				else
					ec_pixmap = GETPIXMAP(expanded_term_pixmap, state);
			}
			else {
				if (p->next)
					ec_pixmap = GETPIXMAP(contracted_pixmap, state);
				else
					ec_pixmap = GETPIXMAP(contracted_term_pixmap, state);
			}
		} else {
			if (p->next)
				ec_pixmap = GETPIXMAP(tree_nochild_pixmap, state);
			else
				ec_pixmap = GETPIXMAP(tree_nochild_term_pixmap, state);
		}
	} else {
		if (p->next)
			ec_pixmap = GETPIXMAP(tree_nochild_pixmap, state);
		else
			ec_pixmap = GETPIXMAP(tree_nochild_term_pixmap, state);
	}

	pkgCache::DepIterator * D = (pkgCache::DepIterator *) (p->Extra);
	if (strcmp(D->DepType(), "Depends") == 0)
		icon = GETPIXMAP(depends_pixmap, state);
	else if (strcmp(D->DepType(), "Pre-Depends") == 0)
		icon = GETPIXMAP(depends_pixmap, state);
	else if (strcmp(D->DepType(), "Recommends") == 0)
		icon = GETPIXMAP(recommends_pixmap, state);
	else if (strcmp(D->DepType(), "Suggests") == 0)
		icon = GETPIXMAP(suggests_pixmap, state);
	else if (strcmp(D->DepType(), "Replaces") == 0)
		icon = GETPIXMAP(replaces_pixmap, state);
	else if (strcmp(D->DepType(), "Conflicts") == 0)
		icon = GETPIXMAP(conflicts_pixmap, state);

	gdk_draw_pixmap(bpixmap, gc, ec_pixmap, 0, 0,
			((p->increment) * GTKUI_PKG_INDENT) + 5, 
			(ypos + 1) * vspacing, 
			GTKUI_TREE_ICON_SIZE, GTKUI_TREE_ICON_SIZE);

	gdk_draw_pixmap(bpixmap, gc, icon, 0, 0,
			((p->increment) * GTKUI_PKG_INDENT) + 5 + GTKUI_TREE_ICON_SIZE + 5,
			(ypos + 1) * vspacing + 2, 
			GTKUI_PKG_ICON_SIZE, GTKUI_PKG_ICON_SIZE);

	gdk_draw_string(bpixmap, treefont, gc,
			((p->increment) * GTKUI_PKG_INDENT) + 5 + GTKUI_TREE_ICON_SIZE + 5 + GTKUI_PKG_ICON_SIZE + 5,
			(ypos + 2) * vspacing - 6, 
			p->TargetPkg.Name());

	pkgCache::VerIterator V, InstV;
	pkgCache::PkgIterator I = p->TargetPkg;
	V = (*Cache)[I].CandidateVerIter(*Cache);
	InstV = (*Cache)[I].InstVerIter(*Cache);

	gdk_draw_string(bpixmap, treefont, gc, columnoffsets[1] + 3, (ypos + 2) * vspacing - 6, (*Cache)[I].Status == 1 ? "Installed" : ((InstV.end() == false || (*Cache)[I].Delete() == true) && (*Cache)[I].NewInstall() == false ? "Upgraded" : "Kept")); 

	gdk_draw_string(bpixmap, treefont, gc, columnoffsets[2] + 3, (ypos + 2) * vspacing - 6, ((*Cache)[I].InstBroken()? "Broken" : ((*Cache)[I].  NewInstall()? "Install" : ((*Cache)[I].  Upgrade()?  "Upgrade" : ((*Cache)[I].  Downgrade()?  "Delete" : ((*Cache) [I].  Install()?  "Install" : ( (*Cache) [I].  Delete ()? ( ( (*Cache) [I].  iFlags & pkgDepCache:: Purge) ? "Purge" : "Remove") : ((*Cache)[I].iFlags & pkgDepCache::ReInstall) ? "Reinstall" : "Keep")))))));

	gdk_draw_string(bpixmap, treefont, gc, columnoffsets[3] + 3, (ypos + 2) * vspacing - 6,
			(V.end()? "" : V.VerStr()));
	gdk_draw_string(bpixmap, treefont, gc, columnoffsets[4] + 3, (ypos + 2) * vspacing - 6,
			(InstV.end()? "" : InstV.VerStr()));
	gdk_draw_string(bpixmap, treefont, gc, columnoffsets[5] + 3, (ypos + 2) * vspacing - 6,
	   (V.end() ? "" : Cache->GetPkgRecs().Lookup(V.FileList()).ShortDesc().c_str()));

	PackageTree::Item *q = p;
	int X = p->increment * GTKUI_PKG_INDENT;
	while (q->parent && X > 0)
	{
		X -= GTKUI_PKG_INDENT;

		if (q->parent->next)
			gdk_draw_line(bpixmap, gc,
				/* x1, y1 */ X + 7, (ypos + 1) * vspacing,
				/* x2, y2 */ X + 7, ((ypos + 2) * vspacing) - 1);

		q = q->parent;
	}
}

void GtkUI::PkgTreeWidget::Drawline(::PackageTree::Item * p, int ypos, bool prelight)
{
	SelectState state = Normal;

	if (p == NULL) {
		g_print("package passed to Drawline is NULL\n");
		return;
	}

	if (prelight)
		state = Prelight;
	else if (p == screen.cs)
		state = Selected;
	else if (p->IsMatched())
		state = Matched;

	if (p->IsDepend() && p->IsExpanded())
		DrawPackage(p, state, ypos);

	switch (p->Type)
	{
	case ::PackageTree::Item::Div_State:
	case ::PackageTree::Item::Div_Sect:
	case ::PackageTree::Item::Div_Pri:
	case ::PackageTree::Item::Div_Custom:
		DrawHeader(p, state, ypos);
		break;
	case ::PackageTree::Item::Pkg:
		DrawPackage(p, state, ypos);
		break;
	case ::PackageTree::Item::Dep:
		DrawDepend(p, state, ypos);
		break;
	default:		/* shouldn't happen */
		NOTREACHED("Unknown package type");
		break;
	}
}

void GtkUI::PkgTreeWidget::MoveCursor(int delta, bool prelight)
{
	long current = screen._cursor - screen._boundstart;
	long newcursor = current + delta;
	::PackageTree::Iterator prevcursor = screen.cs;

	if (delta == 0)
		return;		// well that was easy....

	//g_print("Delta is %d\n", delta);

	if (delta < 0)
	{
		while (delta < 0)
		{
			screen._cursor--;
			screen.cs--;
			delta++;
		}
	}
	else
	{
		while (delta > 0)
		{
			screen._cursor++;
			screen.cs++;
			delta--;
		}
	}

	Drawline(prevcursor, current, false);
	Drawline(screen.cs, newcursor, prelight);

	/* tell gtk to update the two lines we updated */
	GdkRectangle update_rect;
	update_rect.x = 0;
	update_rect.y = (current + 1) * vspacing;
	update_rect.width = theTree->allocation.width;
	update_rect.height = vspacing;
	gtk_widget_draw(theTree, &update_rect);

	update_rect.x = 0;
	update_rect.y = (newcursor + 1) * vspacing;
	update_rect.width = theTree->allocation.width;
	update_rect.height = vspacing;
	gtk_widget_draw(theTree, &update_rect);

	gtk_adjustment_set_value(GTK_ADJUSTMENT(screen.content_adj), screen._cursor);
}

void GtkUI::PkgTreeWidget::DependToggle()
{
	DefaultKeys::depend_toggle(&screen);
	UpdateAdjustment();
	QueueRedraw();
}

GtkWidget *GtkUI::PkgTreeWidget::MakeContextMenu(::PackageTree::Item * p)
{
	/* TODO: Need to make this not leak like a sieve ... */
	pkgCache::VerIterator InstV;
	InstV = (*Cache)[p->TargetPkg].InstVerIter(*Cache);
	GtkWidget *menu = gtk_menu_new();

	GtkWidget *pkgname = gtk_menu_item_new_with_label(p->TargetPkg.Name());
	gtk_menu_append(GTK_MENU(menu), pkgname);
	gtk_widget_set_sensitive(pkgname, FALSE);
	gtk_widget_show(pkgname);
	GtkWidget *sepline = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), sepline);
	gtk_widget_show(sepline);

	GtkWidget *item;

	// marked for install or delete, can keep
	if ((*Cache)[p->TargetPkg].Install() || (*Cache)[p->TargetPkg].Delete())
	{
		item = gtk_menu_item_new_with_label("Keep");
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(MarkKeep),
				   (gpointer) this);
		gtk_widget_show(item);
	}

	if (InstV.end() == false)
	{
		if ((*Cache)[p->TargetPkg].Upgrade() == false
		    && (*Cache)[p->TargetPkg].Status == 1)
		{
			item = gtk_menu_item_new_with_label("Upgrade");
			gtk_menu_append(GTK_MENU(menu), item);
			gtk_signal_connect(GTK_OBJECT(item), "activate",
					   GTK_SIGNAL_FUNC(MarkInstall), (gpointer) this);
			gtk_widget_show(item);
		}

		if ((*Cache)[p->TargetPkg].Delete() == false
		    && (*Cache)[p->TargetPkg].NewInstall() == false)
		{
			if (((*Cache)[p->TargetPkg].iFlags & pkgDepCache::ReInstall) == 0)
			{
				item = gtk_menu_item_new_with_label("Reinstall");
				gtk_menu_append(GTK_MENU(menu), item);
				gtk_signal_connect(GTK_OBJECT(item), "activate",
					   GTK_SIGNAL_FUNC(MarkReinstall), (gpointer) this);
				gtk_widget_show(item);
			}
			else
			{
				item = gtk_menu_item_new_with_label("Keep");
				gtk_menu_append(GTK_MENU(menu), item);
				gtk_signal_connect(GTK_OBJECT(item), "activate",
					GTK_SIGNAL_FUNC(MarkKeep), (gpointer)this);
				gtk_widget_show(item);
			}
			item = gtk_menu_item_new_with_label("Remove");
			gtk_menu_append(GTK_MENU(menu), item);
			gtk_signal_connect(GTK_OBJECT(item), "activate",
					   GTK_SIGNAL_FUNC(MarkRemove), (gpointer) this);
			gtk_widget_show(item);
			item = gtk_menu_item_new_with_label("Purge");
			gtk_menu_append(GTK_MENU(menu), item);
			gtk_signal_connect(GTK_OBJECT(item), "activate",
					   GTK_SIGNAL_FUNC(MarkPurge), (gpointer) this);
			gtk_widget_show(item);
		}
		else if ((*Cache)[p->TargetPkg].Delete() == true
			 && !((*Cache)[p->TargetPkg].iFlags & pkgDepCache::Purge))
		{
			item = gtk_menu_item_new_with_label("Purge");
			gtk_menu_append(GTK_MENU(menu), item);
			gtk_signal_connect(GTK_OBJECT(item), "activate",
					   GTK_SIGNAL_FUNC(MarkPurge), (gpointer) this);
			gtk_widget_show(item);
		}
	}
	else if ((*Cache)[p->TargetPkg].Delete() == false)
	{
		item = gtk_menu_item_new_with_label("Install");
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_signal_connect(GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(MarkInstall),
				   (gpointer) this);
		gtk_widget_show(item);
	}

	return menu;
}

void GtkUI::PkgTreeWidget::QueueRedraw()
{
	if (redrawqueued == false) {
		idleid = gtk_idle_add(ProcessRedraw, this);
		redrawqueued = true;
	}
}

gint GtkUI::PkgTreeWidget::ProcessRedraw(gpointer data)
{
	GtkUI::PkgTreeWidget *tree = (GtkUI::PkgTreeWidget *) data;

	tree->Redraw(true);
	tree->redrawqueued = false;

	return FALSE;
}

void GtkUI::PkgTreeWidget::Redraw(bool immediate)
{
	// blank out the pixmap
	gdk_draw_rectangle(bpixmap, theTree->style->white_gc, TRUE, 0, 0,
			   theTree->allocation.width, theTree->allocation.height);

	DrawHeading();

	unsigned int i = 0;
	::PackageTree::Iterator T(screen.bs);

	for (; !T.Invalid(); i++, T++)
	{
		Drawline(T, i);
		if (T == screen.be)
			break;
	}

	//printf("Drew %d packages\n", i);
	if (resizingcolumn > 0) {
		gdk_draw_line(bpixmap, theTree->style->black_gc,
			columnoffsets[resizingcolumn], 0,
			columnoffsets[resizingcolumn], theTree->allocation.height);
	}

	/* Tell it to redraw the whole widget */
	if (immediate) {
		GdkRectangle update_rect;
		update_rect.x = 0;
		update_rect.y = 0;
		update_rect.width = theTree->allocation.width;
		update_rect.height = theTree->allocation.height;

		gtk_widget_draw(theTree, &update_rect);
	}

	/* Had to remove... for some reason this broke things
	   with more than one window... returning FALSE from the 
		 ProcessRedraw function has the same effect..and works! :) */
	// if (redrawqueued) gtk_idle_remove(idleid);
}

