/* $Id: pkgtreewidget.h,v 1.5 2001/03/24 08:08:32 ltd Exp $ */
#ifndef _gtkui_pkgtree_H
#define _gtkui_pkgtree_H

#include "gtkui.h"

#include <list/pkgtree.h>

#define GTKUI_PKGTREE_COLUMNS 6

class GtkUI::PkgTreeWidget 
{
	friend class GtkUI::Screen;
	friend class GtkUI::SortUI;

protected:
	enum SelectState {
		Normal,		/* not selected */
		Prelight,	/* mouse over */
		Selected,	/* selected */
		Matched		/* matched on a search */
	};

	static GdkPixmap *package_pixmap[4];
	static GdkPixmap *divider_pixmap[4];
	static GdkPixmap *depends_pixmap[4];
	static GdkPixmap *conflicts_pixmap[4];
	static GdkPixmap *replaces_pixmap[4];
	static GdkPixmap *recommends_pixmap[4];
	static GdkPixmap *suggests_pixmap[4];
	static GdkPixmap *expanded_pixmap[4];
	static GdkPixmap *contracted_pixmap[4];
	static GdkPixmap *expanded_term_pixmap[4];
	static GdkPixmap *contracted_term_pixmap[4];
	static GdkPixmap *tree_nochild_term_pixmap[4];
	static GdkPixmap *tree_nochild_pixmap[4];

	/* parent widget */
	GtkUI::Screen &screen;

	GdkGC *normal_gc, *prelight_gc, *selected_gc, *matched_gc;
	GdkPixmap *bpixmap; /* Backing pixmap */
	GdkFont *treefont;
	GtkAdjustment *adjustment;

	GtkWidget *theTree;

	int vspacing;
	bool redrawqueued;
	guint idleid;

	static gchar *titles[];
	gint columnoffsets[GTKUI_PKGTREE_COLUMNS + 1];
	float realcoloffsets[GTKUI_PKGTREE_COLUMNS + 1];
	gint resizingcolumn;

	void SetVAdjustment(GtkAdjustment *a) { adjustment = a; }
	GtkAdjustment *GetVAdjustment() { return adjustment; }

	void SetupGCs();

	void ColumnResize(gint x);
	void UpdateAdjustment();
	void ScrollEvent(GtkAdjustment *);
	
	void DrawHeading();

	void BlankLine(SelectState, gint);
	void DrawHeader(::PackageTree::Item *, SelectState, gint);
	void DrawPackage(::PackageTree::Item *, SelectState, gint);
	void DrawDepend(::PackageTree::Item *, SelectState, gint);
	void Drawline(::PackageTree::Item *, int, bool = false);

	void PrintPackage(::PackageTree::Item *);
	void MoveCursor(int, bool = false);
	void DependToggle();

	GtkWidget *MakeContextMenu(::PackageTree::Item *);

	static gint ProcessRedraw(gpointer);
	
	// event handlers
	static gint Resize(GtkWidget *, GdkEventConfigure *, gpointer);
	static gint Expose(GtkWidget *, GdkEventExpose *, gpointer);
	static void Realize(GtkWidget *, gpointer);
	static gboolean ButtonPress(GtkWidget *, GdkEventButton *, gpointer);
	static gboolean ButtonRelease(GtkWidget *, GdkEventButton *, gpointer);
	static gboolean MotionNotify(GtkWidget *, GdkEventMotion *, gpointer);
	static gboolean LeaveNotify(GtkWidget *, GdkEventCrossing *, gpointer);

	// context menu callbacks
	static void MarkInstall(GtkMenuItem *, gpointer);
	static void MarkReinstall(GtkMenuItem *, gpointer);
	static void MarkUpgrade(GtkMenuItem *, gpointer);
	static void MarkRemove(GtkMenuItem *, gpointer);
	static void MarkPurge(GtkMenuItem *, gpointer);
	static void MarkHold(GtkMenuItem *, gpointer);
	static void MarkKeep(GtkMenuItem *, gpointer);

public:
	PkgTreeWidget(GtkUI::Screen &);
	~PkgTreeWidget();

	// Have to do this--
	int DoPrelight(int y_ref);
	void RemovePrelight();

	GtkWidget *GetTree() { return theTree; }

	void QueueRedraw();
	void Redraw(bool immediate = true);

	int LinesInView();
};

#endif
