/* $Id: screen.cc,v 1.22 2001/05/14 06:57:32 ltd Exp $ */
#include <std.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <func/apt.h>
#include <func/config.h>
#include <func/dependency.h>
#include <func/pkgchecks.h>
#include <init/ops.h>
#include <list/screens.h>
#include <parsers/strutl.h>
#include "about.h"
#include "screen.h"
#include "menu.h"
#include "pkgtreewidget.h"
#include "util.h"
#include "sortui.h"
#include "srcedit.h"
#include "pixmaps/installsm.xpm"
#include "pixmaps/removesm.xpm"
#include "pixmaps/purgesm.xpm"
#include "pixmaps/upgradesm.xpm"
#include "pixmaps/distupgradesm.xpm"
#include "pixmaps/locatesm.xpm"
#include "pixmaps/filtersm.xpm"
#include "pixmaps/completesm.xpm"
#include "pixmaps/cancelsm.xpm"

using namespace GtkUtil;

const char **install_xpm = const_cast < const char **>(installsm_xpm);
const char **remove_xpm = const_cast < const char **>(removesm_xpm);
const char **purge_xpm = const_cast < const char **>(purgesm_xpm);
const char **upgrade_xpm = const_cast < const char **>(upgradesm_xpm);
const char **distupgrade_xpm = const_cast < const char **>(distupgradesm_xpm);
const char **locate_xpm = const_cast < const char **>(locatesm_xpm);
const char **filter_xpm = const_cast < const char **>(filtersm_xpm);
const char **complete_xpm = const_cast < const char **>(completesm_xpm);
const char **cancel_xpm = const_cast < const char **>(cancelsm_xpm);

// titles
gchar *GtkUI::Screen::candidateVerListTitles[] = { "Element", "Value" };
gchar *GtkUI::Screen::installedVerListTitles[] = { "Element", "Value" };
gchar *GtkUI::Screen::availableVersTreeTitles[] =
	{ "Package Version", "Dependencies & Provides" };

GtkUI::Screen::Screen(::Screen::ScrInit mode, ::Screen::ScrFunc func, GtkUI & _ui)
	: ::Screen(mode, func), ui(_ui), window(0), modal(false)
{
	InvalidateIterators();

	// TODO: read from config
	descwinshown = true;

	// The window
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	ui.CreateKeyInterface(window);
	gtk_window_set_title(GTK_WINDOW(window), PACKAGE " " VERSION);
	gtk_window_set_default_size(GTK_WINDOW(window), _Conf.GetInt("deity.gtk.window.main.xsize"), _Conf.GetInt("deity.gtk.window.main.ysize"));
	gtk_widget_realize(window);

	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(destroy), this);

	gtk_signal_connect(GTK_OBJECT(window), "key_press_event", GTK_SIGNAL_FUNC(KeyPress), this);

	mwVBox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), mwVBox);

	// Draw the menu
	GtkWidget *menu = DrawMenu();
	gtk_box_pack_start(GTK_BOX(mwVBox), menu, FALSE, FALSE, 0);

	// Draw the toolbar
	GtkWidget *toolbar = DrawToolbar();
	gtk_box_pack_start(GTK_BOX(mwVBox), toolbar, FALSE, FALSE, 0);

	GtkWidget *mainscr = DrawScreen();
	gtk_box_pack_start(GTK_BOX(mwVBox), mainscr, TRUE, TRUE, 0);

	// Draw the status bar
	status_bar = gtk_statusbar_new();
	gtk_box_pack_start(GTK_BOX(mwVBox), status_bar, FALSE, FALSE, 0);
	guint context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(status_bar), "Deity Status Bar");
	gtk_statusbar_push(GTK_STATUSBAR(status_bar), context_id, "Welcome to Deity!");
	
	// Show everything!
	gtk_widget_show_all(window);
}

GtkUI::Screen::~Screen()
{
	gtk_widget_hide(window);
	delete pkgtree;

	if (modal)
		gtk_main_quit();
}

void GtkUI::Screen::Show(bool ismodal)
{
	modal = ismodal;
}

void GtkUI::Screen::AddToolbarIcon(GtkWidget * toolbar,
				   GtkSignalFunc callback, gpointer callback_data,
				   const char *labeltxt, const char **xpm_data,
				   const char *tooltips)
{
	GtkWidget *icon = NULL;

	if (xpm_data != NULL)
	{
		icon = ImageFromXPM((char **) xpm_data);
	}

	
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), labeltxt, tooltips, NULL,
				icon, callback, callback_data);
}

void GtkUI::Screen::ShowDescription(bool flag)
{

}

GtkWidget *GtkUI::Screen::DrawScreen(bool ShowDesc)
{
	GtkWidget *vpane = gtk_vpaned_new();
	gtk_paned_set_position(GTK_PANED(vpane), _Conf.GetInt("deity.gtk.window.main.ysize") - 200);
	
	GtkWidget *mypkgtree = DrawContent();

	gtk_paned_pack1(GTK_PANED(vpane), mypkgtree, FALSE, TRUE);
	GtkWidget *hpane = gtk_hpaned_new();
	gtk_paned_pack2(GTK_PANED(vpane), hpane, TRUE, TRUE);
	gtk_paned_set_position(GTK_PANED(hpane), 550);

	GtkWidget *scrollwin = gtk_scrolled_window_new(NULL, NULL);
	gtk_paned_pack1(GTK_PANED(hpane), scrollwin, FALSE, TRUE);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	GtkWidget *viewport = gtk_viewport_new(NULL, NULL);
	gtk_container_add(GTK_CONTAINER(scrollwin), viewport);

	description = gtk_label_new("");
	gtk_container_add(GTK_CONTAINER(viewport), description);
	gtk_label_set_justify(GTK_LABEL(description), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment(GTK_MISC(description), 0, 0);
	
	GtkStyle *descstyle = gtk_style_new();
	gdk_font_unref(descstyle->font);
	if ((descstyle->font = gdk_font_load(_Conf.Get("deity.gtk.font.description"))) == NULL) {
		fprintf(stderr, "Cannot load desc style font: %s, trying fixed..\n", _Conf.Get("deity.gtk.font.description"));
		if ((descstyle->font = gdk_font_load("fixed")) == NULL) {
			fprintf(stderr, "Unable to load font fixed, exiting..\n");
			exit(0);
		}
	}

	gtk_widget_set_style(description, descstyle);

	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
	gtk_paned_pack2(GTK_PANED(hpane), vbox, TRUE, TRUE);

	GtkWidget *toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
	gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);
	gtk_toolbar_set_space_size(GTK_TOOLBAR(toolbar), 15);
	gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_LINE);
	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);

	gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), 
		GTK_TOOLBAR_CHILD_BUTTON, NULL, "Candidate",
		"Candidate version(s) that can be installed", NULL, NULL, 
		NULL, NULL);

	gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), 
		GTK_TOOLBAR_CHILD_BUTTON, NULL, "Installed",
		"Installed version(s) currently on this system", NULL, NULL, 
		NULL, NULL);

	gtk_toolbar_append_element(GTK_TOOLBAR(toolbar), 
		GTK_TOOLBAR_CHILD_BUTTON, NULL, "Available",
		"Versions available in the archive", NULL, NULL, 
		NULL, NULL);

	gchar *titles[] = { "Attribute", "Value" };
	detailclist = gtk_clist_new_with_titles(2, titles);
	gtk_box_pack_start(GTK_BOX(vbox), detailclist, TRUE, TRUE, 0);
	gtk_clist_set_column_width(GTK_CLIST(detailclist), 0, 60);
	gtk_clist_set_column_width(GTK_CLIST(detailclist), 1, 80);
	
	UpdateDescription();

	return vpane;
}

GtkWidget *GtkUI::Screen::DrawMenu()
{
	GtkItemFactory *itemfac;
	GtkItemFactoryEntry *item;
	MenuItemCallbackData *cb;
	GtkAccelGroup *accel;
	GtkWidget *menubar;
	unsigned int i;

	accel = gtk_accel_group_new();
	itemfac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel);

	GtkWidget *menuHandler = gtk_handle_box_new();

	for (i = 0; i < sizeof(menu_items) / sizeof(menu_items[0]); i++)
	{
		item = g_new0(GtkItemFactoryEntry, 1);
		item->path = menu_items[i].path;
		item->accelerator = menu_items[i].accelerator;
		item->callback_action = menu_items[i].callback_action;
		item->item_type = menu_items[i].item_type;
		if (menu_items[i].func == 0 && menu_items[i].scr_func == 0)
		{
			gtk_item_factory_create_item(itemfac, item, NULL, 2);
		}
		else
		{
			item->callback = (GtkItemFactoryCallback) MnuDispatch;
			cb = g_new0(MenuItemCallbackData, 1);
			cb->obj = this;
			cb->func = menu_items[i].func;
			cb->scr_func = menu_items[i].scr_func;
			gtk_item_factory_create_item(itemfac, item, (gpointer) cb, 2);

		}
	}

	gtk_window_add_accel_group(GTK_WINDOW(window), accel);

	menubar = gtk_item_factory_get_widget(itemfac, "<main>");

	gtk_container_add(GTK_CONTAINER(menuHandler), menubar);

	return menuHandler;
}

GtkWidget *GtkUI::Screen::DrawToolbar()
{
	GtkWidget *toolbarHandler = gtk_handle_box_new();

	GtkWidget *toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
	gtk_container_add(GTK_CONTAINER(toolbarHandler), toolbar);
	gtk_container_set_border_width(GTK_CONTAINER(toolbar), 3);
	gtk_toolbar_set_space_size(GTK_TOOLBAR(toolbar), 0);
	//gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_LINE);

	GTK_WIDGET_SET_FLAGS (toolbar, GTK_CAN_FOCUS);
	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);
	if (func == PkgList) 
	{
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoComplete), this, "Complete", 
			complete_xpm, "Complete pending installs");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoInstall), this, "Inst/Upg", 
			install_xpm, "Install/Upgrade selected packages");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoRemove), this, "Remove", 
			remove_xpm, "Remove selected packages");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoPurge), this, "Purge", 	
			purge_xpm, "Purge selected packages");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoUpgrade), this, "Upgrade All", 
			upgrade_xpm, "Upgrade all  packages");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoDistUpgrade), this, 	
			"UpDist", distupgrade_xpm, "Upgrade to new distribution");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoLocate), this, "Locate", 	
			locate_xpm, "Locate pacakges");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoFilter), this, "Filter", 	
			filter_xpm, "Filter the package list");	
	} else {
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoComplete2), this, "Complete", 
			complete_xpm, "Complete pending installs");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
		AddToolbarIcon(toolbar, GTK_SIGNAL_FUNC(DoCancel), this, "Cancel", 
			cancel_xpm, "Cancel pending installs");
		gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
	}

	return toolbarHandler;
}

GtkWidget *GtkUI::Screen::DrawContent()
{
	GtkWidget *scroller;
	GtkWidget *hbox;
	GtkWidget *frame;

	hbox = gtk_hbox_new(FALSE, 0);

	// Create the scrollbar
	content_adj = gtk_adjustment_new(0, 0, 100, 1, 10, 10);	/* some default values */

	gtk_signal_connect(GTK_OBJECT(content_adj), "value_changed",
			   GTK_SIGNAL_FUNC(ScrollEvent), this);

	scroller = gtk_vscrollbar_new(GTK_ADJUSTMENT(content_adj));

	gtk_box_pack_end(GTK_BOX(hbox), scroller, FALSE, FALSE, 0);

	// Create the tree widget (put it inside a frame
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

	pkgtree = new GtkUI::PkgTreeWidget(*this);
	pkgtree->SetVAdjustment(GTK_ADJUSTMENT(content_adj));

	gtk_container_add(GTK_CONTAINER(frame), pkgtree->GetTree());

	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 3);

	return hbox;
}

void GtkUI::Screen::UpdateDescription()
{
	if (descwinshown == false) return;
	if (cs.Invalid()) return; 

	if (cs->IsDivider())
	{
		gtk_label_set_text(GTK_LABEL(description),
			Packages->GetHeaderDesc(cs).c_str());
	} 
	else if (cs->Version().end() == true)
	{
		gtk_label_set_text(GTK_LABEL(description), "This package has no version");
	}
	else 
	{
		gtk_label_set_text(GTK_LABEL(description),
			Cache->GetPkgRecs().Lookup(cs->TargetPkg.VersionList().FileList()).LongDesc().c_str());
	}
}

/* static event handlers */
void GtkUI::Screen::ScrollEvent(GtkAdjustment * adj, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	screen->pkgtree->ScrollEvent(adj);
}

gboolean GtkUI::Screen::KeyPress(GtkWidget * w, GdkEventKey * event, gpointer data)
{
	/* 
	 *This lets us override the key capturing functions in func/keys.cc for keycodes
	 * that are defined differently than in the "standard" UI
	 */
	GtkUI::Screen *screen = (GtkUI::Screen *) data;
	switch (event->keyval) {
	case GDK_Up: screen->ArrowUp(); break;
	case GDK_Down: screen->ArrowDown(); break;
	case GDK_Left: screen->ArrowLeft(); break;
	case GDK_Right: screen->ArrowRight(); break;
	case GDK_Page_Up: screen->PageUp(); break;
	case GDK_Page_Down: screen->PageDown(); break;
	case GDK_Home: screen->SeekStart(); break;
	case GDK_End: screen->SeekEnd(); break;
	case GDK_Tab: screen->pkgtree->DependToggle(); break;
	}
	return TRUE;
}


gint GtkUI::Screen::delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
{
	/* Change TRUE to FALSE and the main window will be destroyed with
	   * a "delete_event". */

	return (FALSE);
}

void GtkUI::Screen::destroy(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen *_this = static_cast<GtkUI::Screen *>(data);
	GtkUI &gui = _this->ui;

	gui.scrlst.remove((::Screen *) _this);
	
	if (gui.scrlst.length() == 0)
		gui.Shutdown();
}

void GtkUI::Screen::DoComplete(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	DefaultKeys::complete_pending(screen);
}

void GtkUI::Screen::DoComplete2(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	RestrictedKeys::complete(screen);
}

void GtkUI::Screen::DoCancel(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	RestrictedKeys::abort(screen);
}

void GtkUI::Screen::DoInstall(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	GtkUI::PkgTreeWidget::MarkInstall(NULL, screen->pkgtree);
}

void GtkUI::Screen::DoRemove(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	GtkUI::PkgTreeWidget::MarkRemove(NULL, screen->pkgtree);
}

void GtkUI::Screen::DoPurge(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	GtkUI::PkgTreeWidget::MarkPurge(NULL, screen->pkgtree);
}

void GtkUI::Screen::DoUpgrade(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;

	if (pkgAllUpgrade(*Cache) == false)
		capt_show_broken(*Cache);

	screen->pkgtree->Redraw();
}

void GtkUI::Screen::DoDistUpgrade(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	if (pkgDistUpgrade(*Cache) == false)
		capt_show_broken(*Cache);

	screen->pkgtree->Redraw();
}

void GtkUI::Screen::DoLocate(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	DefaultKeys::search(screen);
}

void GtkUI::Screen::DoFilter(GtkWidget * widget, gpointer data)
{
	GtkUI::Screen * screen = (GtkUI::Screen *) data;
	filter_add_single(screen);
}

// Menu functions
void GtkUI::Screen::MnuAbout(::Screen *)
{
	About about;
}

void GtkUI::Screen::MnuDispatch(GtkWidget * widget, gpointer closure, guint type)
{
	MenuItemCallbackData *cb = (MenuItemCallbackData *) closure;
	if (cb) {
		if (cb->func) {
			(*(cb->func))(cb->obj);
		} else if (cb->scr_func) {
			((cb->obj)->*(cb->scr_func)) ();
		}
	}
}


void GtkUI::Screen::InvalidateIterators()
{
	_boundstart = _boundend = _cursor = _prelight = 0;
	bs = (::PackageTree::Item *) 0;
	be = (::PackageTree::Item *) 0;
	cs = (::PackageTree::Item *) 0;
	pl = (::PackageTree::Item *) 0;
}

void GtkUI::Screen::BoundsReset()
{
	if (pkgtree == 0)
		return;

	int I = 0;
	::PackageTree::Iterator T(*Packages);
	screenlength = pkgtree->LinesInView();

	if (Packages->Length() <= screenlength)
	{
		_boundstart = 0;
		_boundend = Packages->Length() - 1;
	}
	else
	{
		_boundend = _boundstart + screenlength - 1;

		while (_boundend >= Packages->Length())
		{
			_boundend--;
			_boundstart--;
		}

		if (_boundstart < 0)
			_boundstart = 0;
	}

	if (_cursor > _boundend)
		_cursor = _boundend;

	if (_cursor < 0)
		_cursor = 0;

	_prelight = _cursor;

	while (I < _boundstart)
	{
		T++;
		I++;
	}

	bs = T;

	while (I < _cursor)
	{
		T++;
		I++;
	}

	cs = T;
	pl = T;

	while (I < _boundend)
	{
		T++;
		I++;
	}

	be = T;

	while (be->IsHidden())
		be--;
}

void GtkUI::Screen::ScrollUp() 
{
	bs--;
	_boundstart--;
	be--;
	_boundend--;

	if (!pl.Invalid())
		GetTree().DoPrelight(_prelight);
}

void GtkUI::Screen::ScrollDown()
{
	bs++;
	_boundstart++;
	be++;
	_boundend++;

	if (!pl.Invalid())
		GetTree().DoPrelight(_prelight);
}

void GtkUI::Screen::RedrawRow(int row, ::PackageTree::Item *p)
{
	GetTree().Drawline(p, row, false);
}

void GtkUI::Screen::Redraw()
{
	GetTree().Redraw();
}

void GtkUI::Screen::RedrawTree()
{
	GetTree().Redraw();
}

bool GtkUI::Screen::IsDescWindowShown()
{
	return descwinshown;
}

void GtkUI::Screen::DescWindowShow()
{
	ShowDescription(true);
	descwinshown = true;
}

void GtkUI::Screen::DescWindowHide()
{
	ShowDescription(false);
	descwinshown = false;
}

void GtkUI::Screen::DrawNextScreen() {}
void GtkUI::Screen::DrawPrevScreen() {}

void GtkUI::Screen::SeekNextMatched() {}
void GtkUI::Screen::DividerNext() {}
void GtkUI::Screen::DividerPrev() {}

GtkWidget *GtkUI::Screen::DrawSearchDialog()
{
	static gchar *advpkgoptmnulbls[] = { "package name", "candidate version", "install version", "short description", "long description", "maintainer", "install size" };
	static gchar *advopoptmnulbls[] = { "is equal to", "is not equal to", "is less than", "is greater than", "contains", "does not contain" };

	int i;
	unsigned int j;
	GtkWidget *dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), "Search...");
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
	gtk_window_set_policy(GTK_WINDOW(dialog), TRUE, TRUE, FALSE);

	GtkWidget *dlgvbox = GTK_DIALOG(dialog)->vbox;
	
	GtkWidget *notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(dlgvbox), notebook, FALSE, TRUE, 0);

	// First page -- simple search
	GtkWidget *simplepage = gtk_vbox_new(FALSE, 18);
	gtk_container_add(GTK_CONTAINER(notebook), simplepage);
	gtk_container_set_border_width(GTK_CONTAINER(simplepage), 20);

	gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook), 
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 0),
		gtk_label_new("Simple search"));

	gtk_box_pack_start(GTK_BOX(simplepage), gtk_label_new("Search for:"), 
		FALSE, FALSE, 0);
	GtkWidget *simpleentry = gtk_entry_new();
	gtk_object_set_data(GTK_OBJECT(dialog), "simple_entry", simpleentry);
	gtk_box_pack_start(GTK_BOX(simplepage), simpleentry,
		FALSE, FALSE, 0);
	
	GtkWidget *simpleopthbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(simplepage), simpleopthbox, TRUE, TRUE, 0);

	GtkWidget *simple_optregex = gtk_check_button_new_with_label("Use regex");
	gtk_object_set_data(GTK_OBJECT(dialog), "simple_optregex", simple_optregex);
	gtk_box_pack_start(GTK_BOX(simpleopthbox), simple_optregex, FALSE, FALSE, 0);
	GtkWidget *simple_optcase = gtk_check_button_new_with_label("Case insensitive");
	gtk_object_set_data(GTK_OBJECT(dialog), "simple_optcase", simple_optcase);
	gtk_box_pack_start(GTK_BOX(simpleopthbox), simple_optcase, FALSE, FALSE, 0);
	GtkWidget *simple_optfull = gtk_check_button_new_with_label("Search descriptions");
	gtk_object_set_data(GTK_OBJECT(dialog), "simple_optfull", simple_optfull);
	gtk_box_pack_start(GTK_BOX(simpleopthbox), simple_optfull, FALSE, FALSE, 0);

	
	// Second page -- Advanced search
	GtkWidget *advpage = gtk_table_new(6, 5, FALSE);
	gtk_container_add(GTK_CONTAINER(notebook), advpage);

	gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook), 
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 1),
		gtk_label_new("Advanced search"));

	// create the field option menus
	for (i = 0; i < 6; i++) {
		GtkWidget *optmenu;
		GtkWidget *optmenu_mnu;

		if (i != 0) {
			GSList *radiogrp = NULL;
			GtkWidget *radioand = gtk_radio_button_new_with_label(radiogrp, "and");
			radiogrp = gtk_radio_button_group(GTK_RADIO_BUTTON(radioand));
			GtkWidget *radioor = gtk_radio_button_new_with_label(radiogrp, "or");
			radiogrp = gtk_radio_button_group(GTK_RADIO_BUTTON(radioor));

			gtk_table_attach(GTK_TABLE(advpage), radioand, 
				0, 1, i, i+1, 
				(GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);
			gtk_table_attach(GTK_TABLE(advpage), radioor, 
				1, 2, i, i+1,
				(GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);
		}

		optmenu = gtk_option_menu_new();
		optmenu_mnu = gtk_menu_new();
		
		for (j = 0; j < sizeof(advpkgoptmnulbls) / sizeof(advpkgoptmnulbls[0]); j++) {
			GtkWidget *menuitem = gtk_menu_item_new_with_label(advpkgoptmnulbls[j]);
			gtk_menu_append(GTK_MENU(optmenu_mnu), menuitem);
			gtk_widget_show(menuitem);
		}
		gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), optmenu_mnu);
		gtk_table_attach(GTK_TABLE(advpage), optmenu, 
			2, 3, i, i+1, 
			(GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);

		optmenu = gtk_option_menu_new();
		optmenu_mnu = gtk_menu_new();
		
		for (j = 0; j < sizeof(advopoptmnulbls) / sizeof(advopoptmnulbls[0]); j++) {
			GtkWidget *menuitem = gtk_menu_item_new_with_label(advopoptmnulbls[j]);
			gtk_menu_append(GTK_MENU(optmenu_mnu), menuitem);
			gtk_widget_show(menuitem);
		}
		gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), optmenu_mnu);
		gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu), 0);
		gtk_table_attach(GTK_TABLE(advpage), optmenu, 
			3, 4, i, i+1,
			(GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);

		GtkWidget *entry = gtk_entry_new();
		gtk_table_attach(GTK_TABLE(advpage), entry,
			4, 5, i, i+1, 
			(GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
			(GtkAttachOptions)0, 0, 0);
	}

	
	GtkWidget *dlgactarea = GTK_DIALOG(dialog)->action_area;

	gtk_container_set_border_width(GTK_CONTAINER(dlgactarea), 10);
	GtkWidget *btnOK = gtk_button_new_with_label("OK");
	gtk_box_pack_start(GTK_BOX(dlgactarea), btnOK, FALSE, TRUE, 0);
	GtkWidget *btnCancel = gtk_button_new_with_label("Cancel");
	gtk_box_pack_start(GTK_BOX(dlgactarea), btnCancel, FALSE, TRUE, 0);

	gtk_signal_connect(GTK_OBJECT(btnOK), "clicked",
		GTK_SIGNAL_FUNC(SearchRun), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(btnCancel), "clicked",
		GTK_SIGNAL_FUNC(SearchCancel), this);

	return dialog;
}

void GtkUI::Screen::SearchEntry(char **buf, unsigned long *options) 
{
	**buf = 0;
	*options = 0;

	GtkWidget *dialog = DrawSearchDialog();

	searchproceed = false;

	gtk_widget_show_all(dialog);
	gtk_main();
	if (searchproceed) {
		GtkWidget *entry = 
			(GtkWidget *)gtk_object_get_data(GTK_OBJECT(dialog),
				"simple_entry");
		GtkWidget *optregex = 
			(GtkWidget *)gtk_object_get_data(GTK_OBJECT(dialog),
				"simple_optregex");
		GtkWidget *optcase = 
			(GtkWidget *)gtk_object_get_data(GTK_OBJECT(dialog),
				"simple_optcase");
		GtkWidget *optfull = 
			(GtkWidget *)gtk_object_get_data(GTK_OBJECT(dialog),
				"simple_optfull");

		strcpy(*buf, gtk_entry_get_text(GTK_ENTRY(entry)));
		if (GTK_TOGGLE_BUTTON(optregex)->active)
			*options ^= SEARCH_REGEX;
		if (GTK_TOGGLE_BUTTON(optcase)->active)
			*options ^= SEARCH_CASE;
		if (GTK_TOGGLE_BUTTON(optfull)->active)
			*options ^= SEARCH_DESC;

		g_print("Searching for %s\n", *buf);
			
	} else {
		g_print("Search canceled\n");
	}

	gtk_widget_destroy(dialog);
}

void GtkUI::Screen::SearchRun(GtkButton *button, gpointer data)
{
	GtkUI::Screen *screen = (GtkUI::Screen *)data;
	screen->searchproceed = true;
	gtk_main_quit();
}

void GtkUI::Screen::SearchCancel(GtkButton *button, gpointer data)
{
	GtkUI::Screen *screen = (GtkUI::Screen *)data;
	screen->searchproceed = false;
	gtk_main_quit();
}

void GtkUI::Screen::SearchHelp() 
{
}

void GtkUI::Screen::SortSelectMethod() 
{
	SortUI sortui(*this);
	sortui.Show();
}

void GtkUI::Screen::EditSources()
{
	SrcEditor editor(*this);
	editor.Run();
}

void GtkUI::Screen::HelpDialog(void) 
{
	GtkWidget *dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 640, 480);

	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(DeleteEvent), NULL);
	gtk_signal_connect(GTK_OBJECT(dialog), "destroy", GTK_SIGNAL_FUNC(Destroy), NULL);

	GtkWidget *vbox = gtk_vbox_new(FALSE, 1);

	gtk_container_border_width(GTK_CONTAINER(vbox), 0);

	gtk_container_add(GTK_CONTAINER(dialog), vbox);

	gchar *titles[] = { "Keypress Binding", "Designated outcome of keypress" };

	GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

	GtkWidget *keylist = gtk_ctree_new_with_titles(2, 0, titles);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), keylist);

	GtkCTreeNode *Parent = NULL;

	for (KeyHandler::List *p = kdriver->ListStart(); p->key != 0; p++)
	{
		if (p->handlefunc == NULL && p->scrfunc == 0)
		{
			gchar *data[] = { stripcolorcodes(p->KeyDesc), "" };

			Parent = gtk_ctree_insert_node(
				GTK_CTREE(keylist),
				NULL, NULL, data, 2,
				NULL, NULL, NULL, NULL,
				FALSE, TRUE);
		}
		else
		{
			gchar *data[] = { p->KeyName, stripcolorcodes(p->KeyDesc) };

			gtk_ctree_insert_node(
				GTK_CTREE(keylist),
				Parent, NULL, data, 2,
				NULL, NULL, NULL, NULL,
				TRUE, TRUE);
		}
	}

	gtk_clist_set_column_width(GTK_CLIST(&(GTK_CTREE(keylist)->clist)), 0, 200);

	gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

	GtkWidget *button = gtk_button_new_with_label("Close");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(WidgetDestroyEvent), dialog);

	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, TRUE, 0);

	gtk_widget_show_all(dialog);

	gtk_main();
} 

void GtkUI::Screen::PageUp()
{
	::Screen::PageUp();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();
}

void GtkUI::Screen::PageDown()
{
	::Screen::PageDown();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();
}

void GtkUI::Screen::ArrowUp()
{
	::Screen::ArrowUp();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();
}

void GtkUI::Screen::ArrowDown()
{
	::Screen::ArrowDown();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();
}

void GtkUI::Screen::ArrowLeft()
{
	::Screen::ArrowLeft();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
}

void GtkUI::Screen::ArrowRight()
{
	::Screen::ArrowRight();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
}

void GtkUI::Screen::SeekStart()
{
	::Screen::SeekStart();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();
}

void GtkUI::Screen::SeekEnd()
{
	::Screen::SeekEnd();
	gtk_adjustment_set_value(GTK_ADJUSTMENT(content_adj), _cursor);
	UpdateDescription();
}

