/* $Id: sortui.cc,v 1.9 2000/11/17 14:56:21 ltd Exp $ */
#include <std.h>
#include "sortui.h"
#include "pkgtreewidget.h"
#include "screen.h"

GtkUI::SortUI::SortUI(GtkUI::Screen &_screen)
	: screen(_screen), window(0), srcrow(-1), dstrow(-1)
{
	gchar *title = "Method";
	gchar *methods[] = { 
		"Name", 
		"Priority",
		"Status", 
		"Archive size", 
		"Installed size", 
		"Type/Section/Area", 
		"Distribution",
		"Suite" };

	window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(window), "Set Sort Methods");
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
	gtk_window_set_modal(GTK_WINDOW(window), TRUE);

	GtkWidget *vbox = gtk_vbox_new(FALSE, 7);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 20);

	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	srclist = gtk_clist_new_with_titles(1, &title);
	gtk_clist_set_shadow_type (GTK_CLIST(srclist), GTK_SHADOW_OUT);
	gtk_clist_set_column_width(GTK_CLIST(srclist), 0, 150);
	gtk_box_pack_start(GTK_BOX(hbox), srclist, TRUE, TRUE, 0);
	gtk_clist_set_auto_sort(GTK_CLIST(srclist), TRUE);
	gtk_signal_connect(GTK_OBJECT(srclist), "select-row",
		GTK_SIGNAL_FUNC(SelectSrcRow), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(srclist), "unselect-row",
		GTK_SIGNAL_FUNC(UnselectSrcRow), (gpointer)this);

	GtkWidget *vbox2 = gtk_vbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 10);
	
	GtkWidget *btnadd = gtk_button_new_with_label("->");
	gtk_signal_connect(GTK_OBJECT(btnadd), "clicked",
		GTK_SIGNAL_FUNC(AddMethod), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(vbox2), btnadd, FALSE, FALSE, 0);
	GtkWidget *btnremove = gtk_button_new_with_label("<-");
	gtk_signal_connect(GTK_OBJECT(btnremove), "clicked",
		GTK_SIGNAL_FUNC(RemoveMethod), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(vbox2), btnremove, FALSE, FALSE, 0);
	GtkWidget *btnmoveup = gtk_button_new_with_label("Move Up");
	gtk_signal_connect(GTK_OBJECT(btnmoveup), "clicked",
		GTK_SIGNAL_FUNC(MoveUp), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(vbox2), btnmoveup, FALSE, FALSE, 0);
	GtkWidget *btnmovedown = gtk_button_new_with_label("Move Down");
	gtk_signal_connect(GTK_OBJECT(btnmovedown), "clicked",
		GTK_SIGNAL_FUNC(MoveDown), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(vbox2), btnmovedown, FALSE, FALSE, 0);

	dstlist = gtk_clist_new_with_titles(1, &title);
	gtk_clist_set_shadow_type (GTK_CLIST(dstlist), GTK_SHADOW_OUT);
	gtk_clist_set_column_width(GTK_CLIST(dstlist), 0, 150);
	gtk_box_pack_start(GTK_BOX(hbox), dstlist, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(dstlist), "select-row",
		GTK_SIGNAL_FUNC(SelectDstRow), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(dstlist), "unselect-row",
		GTK_SIGNAL_FUNC(UnselectDstRow), (gpointer)this);

	GtkWidget *hbox2 = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox2), 8);

	GtkWidget *btnapply = gtk_button_new_with_label("Apply");
	gtk_signal_connect(GTK_OBJECT(btnapply), "clicked",
		GTK_SIGNAL_FUNC(Apply), (gpointer)this);
	gtk_box_pack_start(GTK_BOX(hbox2), btnapply, TRUE, TRUE, 15);

	GtkWidget *btncancel = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(btncancel), "clicked",
		GTK_SIGNAL_FUNC(Cancel), (gpointer)this);

	gtk_box_pack_end(GTK_BOX(hbox2), btncancel, TRUE, TRUE, 15);

	// Add the methods to the source list
	if (!screen.Packages->Sorting->Exists(SortOrderList::PkgName))
		gtk_clist_append(GTK_CLIST(srclist), &methods[0]);
	if (!screen.Packages->Sorting->Exists(SortOrderList::Priority))
		gtk_clist_append(GTK_CLIST(srclist), &methods[1]);
	if (!screen.Packages->Sorting->Exists(SortOrderList::Status))
		gtk_clist_append(GTK_CLIST(srclist), &methods[2]);
	if (!screen.Packages->Sorting->Exists(SortOrderList::Size))
		gtk_clist_append(GTK_CLIST(srclist), &methods[3]);
	if (!screen.Packages->Sorting->Exists(SortOrderList::Instsize))
		gtk_clist_append(GTK_CLIST(srclist), &methods[4]);
	if (!screen.Packages->Sorting->Exists(SortOrderList::Section))
		gtk_clist_append(GTK_CLIST(srclist), &methods[5]);
	if (!screen.Packages->Sorting->Exists(SortOrderList::Distribution))
		gtk_clist_append(GTK_CLIST(srclist), &methods[6]);
	if (!screen.Packages->Sorting->Exists(SortOrderList::Suite))
		gtk_clist_append(GTK_CLIST(srclist), &methods[7]);

	// Add already installed methods to the destination list
	for (SortOrderList::SortIterator S = screen.Packages->Sorting->ListBegin(); S.end() == false; S++)
	{
		switch (S.method())
		{
		case SortOrderList::PkgName:
			gtk_clist_append(GTK_CLIST(dstlist), &methods[0]);
			break;
		case SortOrderList::Priority:
			gtk_clist_append(GTK_CLIST(dstlist), &methods[1]);
			break;
		case SortOrderList::Status:
			gtk_clist_append(GTK_CLIST(dstlist), &methods[2]);
			break;
		case SortOrderList::Size:
			gtk_clist_append(GTK_CLIST(dstlist), &methods[3]);
			break;
		case SortOrderList::Instsize:
			gtk_clist_append(GTK_CLIST(dstlist), &methods[4]);
			break;
		case SortOrderList::Section:
			gtk_clist_append(GTK_CLIST(dstlist), &methods[5]);
			break;
		case SortOrderList::Distribution:
			gtk_clist_append(GTK_CLIST(dstlist), &methods[6]);
			break;
		case SortOrderList::Suite:
			gtk_clist_append(GTK_CLIST(dstlist), &methods[7]);
			break;
		}
	}

}

GtkUI::SortUI::~SortUI()
{
	if (window)
		gtk_widget_destroy(window);
	window = 0;
}

void GtkUI::SortUI::Show()
{
	gtk_widget_show_all(window);
	// enter local event loop
	gtk_main();
}

void GtkUI::SortUI::AddMethod(GtkWidget *widget, gpointer data)
{
	GtkUI::SortUI *ui = (GtkUI::SortUI *)data;
	if (ui->srcrow < 0) return;

	gchar *text;
	gtk_clist_get_text(GTK_CLIST(ui->srclist), ui->srcrow, 0,
		&text);
	gtk_clist_append(GTK_CLIST(ui->dstlist), &text);
	gtk_clist_remove(GTK_CLIST(ui->srclist), ui->srcrow);
}

void GtkUI::SortUI::RemoveMethod(GtkWidget *widget, gpointer data)
{
	GtkUI::SortUI *ui = (GtkUI::SortUI *)data;
	if (ui->dstrow < 0) return;

	gchar *text;
	gtk_clist_get_text(GTK_CLIST(ui->dstlist), ui->dstrow, 0,
		&text);
	gtk_clist_append(GTK_CLIST(ui->srclist), &text);
	gtk_clist_remove(GTK_CLIST(ui->dstlist), ui->dstrow);
}

void GtkUI::SortUI::MoveUp(GtkWidget *widget, gpointer data)
{
	GtkUI::SortUI *ui = (GtkUI::SortUI *)data;
	if (ui->dstrow < 1) return;
	gtk_clist_swap_rows(GTK_CLIST(ui->dstlist), ui->dstrow, ui->dstrow-1);
	ui->dstrow--;
}

void GtkUI::SortUI::MoveDown(GtkWidget *widget, gpointer data)
{
	GtkUI::SortUI *ui = (GtkUI::SortUI *)data;
	gchar *dummy;
	if (ui->dstrow < 0) return;
	if (gtk_clist_get_text(GTK_CLIST(ui->dstlist), ui->dstrow+1, 0, &dummy) == 0) return;
	// should check to see if this is the last row too...
	gtk_clist_swap_rows(GTK_CLIST(ui->dstlist), ui->dstrow, ui->dstrow+1);
	ui->dstrow++;
}

void GtkUI::SortUI::Apply(GtkWidget *, gpointer data)
{
	GtkUI::SortUI *ui = (GtkUI::SortUI *)data;
	gchar *txt;

	ui->screen.Packages->Sorting->Flush();

	for (int i = 0; gtk_clist_get_text(GTK_CLIST(ui->dstlist), i, 0, &txt) != 0; i++) {
		if (strcmp(txt, "Name") == 0) {
			ui->screen.Packages->Sorting->Push(SortOrderList::PkgName);
		} else if (strcmp(txt, "Priority") == 0) {
			ui->screen.Packages->Sorting->Push(SortOrderList::Priority);
		} else if (strcmp(txt, "Status") == 0) {
			ui->screen.Packages->Sorting->Push(SortOrderList::Status);
		} else if (strcmp(txt, "Archive size") == 0) {
			ui->screen.Packages->Sorting->Push(SortOrderList::Size);
		} else if (strcmp(txt, "Installed size") == 0) {
			ui->screen.Packages->Sorting->Push(SortOrderList::Instsize);
		} else if (strcmp(txt, "Type/Section/Area") == 0) {
			ui->screen.Packages->Sorting->Push(SortOrderList::Section);
		} else if (strcmp(txt, "Distribution") == 0) {
			ui->screen.Packages->Sorting->Push(SortOrderList::Distribution);
		} else if (strcmp(txt, "Suite") == 0) {
			ui->screen.Packages->Sorting->Push(SortOrderList::Suite);
		}
	}

	gtk_main_quit();
	gtk_widget_hide(ui->window);

	ui->screen.Packages->reinitialize();
	ui->screen.InvalidateIterators();
	ui->screen.BoundsReset();
	ui->screen.pkgtree->Redraw();
}

void GtkUI::SortUI::Cancel(GtkWidget *, gpointer data)
{
	GtkUI::SortUI *ui = (GtkUI::SortUI *)data;
	gtk_main_quit();
	gtk_widget_hide(ui->window);
}

void GtkUI::SortUI::SelectSrcRow(GtkCList *list, gint row, gint col, GdkEventButton *btn, gpointer data)
{
	GtkUI::SortUI *This = (GtkUI::SortUI *)data;
	This->srcrow = row;
}

void GtkUI::SortUI::UnselectSrcRow(GtkCList *list, gint row, gint col, GdkEventButton *btn, gpointer data)
{
	GtkUI::SortUI *This = (GtkUI::SortUI *)data;
	This->srcrow = -1;
}

void GtkUI::SortUI::SelectDstRow(GtkCList *list, gint row, gint col, GdkEventButton *btn, gpointer data)
{
	GtkUI::SortUI *This = (GtkUI::SortUI *)data;
	This->dstrow = row;
}

void GtkUI::SortUI::UnselectDstRow(GtkCList *list, gint row, gint col, GdkEventButton *btn, gpointer data)
{
	GtkUI::SortUI *This = (GtkUI::SortUI *)data;
	This->dstrow = -1;
}

