/* $Id: filters.cc,v 1.15 2000/12/13 05:46:17 ltd Exp $ */
#include <std.h>
#include <stdio.h>
#include <regex.h>

#include "pkgtree.h"
#include "screens.h"
#include <interface/deityui.h>
#include "filters.h"

FilterList::FilterList(PackageTree *_Owner) : Owner(_Owner)
{
	Head = Tail = NULL;
	nodecount = 0;
}

Filter *FilterList::Push(const char *text, Filter::Target &targ)
{	
	if (text == NULL)
		return NULL;

	if (*text == 0)
		return NULL;

	nodecount += 1;

	if (Head == NULL && Tail == NULL)
	{
		Head = Tail = new Filter(text, targ);
		Tail->next = NULL;
	}
	else
	{
		Tail->next = new Filter(text, targ);
		Tail->next->next = NULL;
		Tail = Tail->next;
	}

	return Tail;
}

void FilterList::Pop(void)
{
	Filter *p;

	nodecount -= 1;

	if (Head == Tail)
	{
		delete Head;
		Head = Tail = NULL;
	}
	else
	{
		for (p = Head; p->next != Tail; p = p->next);
		p->next = p->next->next;
		delete Tail;
		Tail = p;
	}
}

bool FilterList::SimulateApply(void)
{
	Filter *p;
	int filtercount=0;
	int didfilter=0;
	int parsedpkgs=0;

	for (p = Head; p != NULL; p = p->next)
		p->Count = 0;

	for (pkgCache::PkgIterator I = (*Cache)->PkgBegin(); I.end() == false; I++)
	{
		if (I.VersionList().end() == true)
			continue;

		didfilter=0;
		parsedpkgs++;

		for (p = Head; p != NULL; p = p->next)
		{
			if (p->PkgFiltered(I))
				didfilter++;
			else
				p->Count++;
		}

		if (didfilter == nodecount)
			filtercount++;
	}

	if (filtercount == 0 || filtercount == parsedpkgs)
		return false;

	return true;
}

bool FilterList::IsPkgFiltered(pkgCache::PkgIterator I)
{
	for (Filter *F = Head; F != NULL; F = F->next)
		if (F->PkgFiltered(I))
			return true;

	return false;
}

void FilterList::Audit(void)
{
	while (nodecount > 0)
		Pop();
}

void FilterList::Display(void) {} /*
{
	struct Node *p;

	//DeityUI::Dialog *myFilters = CaptUI->CreateDialog();

	myFilters->set_type(DeityUI::Dialog::List_Columned);

	DeityUI::Dialog::Column colid[3];
	DeityUI::Dialog::Row row;

	colid[0] = myFilters->add_column("Id");
	colid[1] = myFilters->add_column("Regex");
	colid[2] = myFilters->add_column("Count");

	for (p = Head; p != NULL; p = p->next)
	{
		row = myFilters.add_row();
		myFilters.add_item(row, colid[0], p->FilterID);
		myFilters.add_item(row, colid[1], p->filtercount);
		myFilters.add_item(row, colid[2], p->String);
		//sprintf(lbuf, " $2#Id: $7#%03ld $!| $2#Matches: $7#%04ld $4 $!| $2#Regex: $7#%s ", p->FilterID, p->filtercount, p->String);
	}

	myFilters->Run();

	CaptUI->RefreshCurrent();
}
*/
/** class Filter **/

#define ___REGFILT(s) if (polarity == positive) \
{ \
	if (regexec(&preg, s, 0, NULL, 0) != 0) \
		WillFilter |= true; \
} else \
{ \
	if (regexec(&preg, s, 0, NULL, 0) == 0) \
		WillFilter |= true; \
}

bool Filter::PkgFiltered(pkgCache::PkgIterator I)
{
	bool WillFilter = false;
	pkgCache::VerIterator V = (*Cache)[I].CandidateVerIter(*Cache);
	pkgCache::VerIterator InstV = (*Cache)[I].InstVerIter(*Cache);

	if (targets & pkgname)
		___REGFILT(I.Name());
	if (targets & pkgsection)
		if (I.Section())
			___REGFILT(I.Section());

	if (V.end() == false)
	{
		if (targets & pkgavailver)
			___REGFILT(V.VerStr());
		if (targets & pkgpriority)
			___REGFILT(V.PriorityType());
		if (targets & pkgshortdesc)
			___REGFILT(Cache->GetPkgRecs().Lookup(V.FileList()).ShortDesc().c_str());
		if (targets & pkglongdesc)
			___REGFILT(Cache->GetPkgRecs().Lookup(V.FileList()).LongDesc().c_str());
	}
	if (InstV.end() == false)
		if (targets & pkginstlver)
			___REGFILT(InstV.VerStr());

	return WillFilter;
}

Filter::Filter(const char *string, Target &targ)
{
	Text = strdup(string);
	ID = 1;
	Count = 0;
	polarity = positive;
	targets = targ;
	regcomp(&preg, string, REG_EXTENDED | REG_ICASE);
}

Filter::Filter()
{
	Text = NULL;
	ID = 1;
	Count = 0;
	polarity = positive;
	targets = (Target) 0;
}

Filter::~Filter()
{
	free(Text);
	regfree(&preg);
}

/** end class Filter */


void filter_revoke_all(Screen *screen)
{
	if (screen->Packages->Filters->Length() == 0)
		return;

	screen->Packages->Filters->Audit();
	screen->Packages->reinitialize();

	screen->BoundsReset();
	screen->Redraw();
}

void filter_display_all(Screen *screen)
{
	if (screen->Packages->Filters->Length() == 0)
		return;

	screen->Packages->Filters->Display();

	screen->Redraw();
}

void filter_change_view(Screen *screen)
{
	if (screen->Packages->Filters->Length() == 0)
		return;

	screen->Packages->reinitialize();

	screen->BoundsReset();
	screen->Redraw();
}

void filter_add_single(Screen *screen)
{
	char *buf;
	Filter::Target targ;

	if (CaptUI->FilterSelect(&buf, &targ) == false)
		return;

	screen->Packages->Filters->Push(buf, targ);

	free(buf);

	if (screen->Packages->Filters->SimulateApply() == false)
	{
		CaptUI->ShowDialog("Filter Error",
			"\n"
			" The effect of your list of filters either: \n"
			"\n"
			" (i)  filters every package \n"
			" (ii) filters no packages \n"
			"\n"
			" Try and refine or expand your regular expression \n");

		screen->Packages->Filters->Pop();

		screen->RefreshCurrent();

		return;
	}
	screen->Packages->apply_filters();
	screen->BoundsReset();

	screen->Redraw();
}


