/* $Id: filters.h,v 1.8 2000/12/13 05:46:17 ltd Exp $ */
#ifndef _filter_H
#define _filter_H

#include <apt-pkg/cacheiterators.h>
#include <regex.h>

class PackageTree;
class Screen;
class Filter;
class FilterList;

class Filter
{
public:
	friend class FilterList;
	enum Target 
	{
		pkgname=(1<<0), 
		pkgavailver=(1<<1), 
		pkginstlver=(1<<2), 
		pkgsection=(1<<3),
		pkgpriority=(1<<4),
		pkgshortdesc=(1<<5), 
		pkglongdesc=(1<<6), 
	};
	enum Polarity
	{
		negative=0, 
		positive=1
	};
	int Count;
	bool PkgFiltered(pkgCache::PkgIterator I);
	inline void setpolarity(Polarity &p) { polarity = p; };
	inline void addtarget(Target &targ) { int(targets) |= targ; };
	Filter(const char *, Target &);
	Filter();
	~Filter();
private:
	int ID;
	char *Text;
	Polarity polarity;
	Target targets;
	regex_t preg;
	Filter *next;
};

class FilterList
{	
protected:
	PackageTree *Owner;
	Filter *Head, *Tail;
	long nodecount;
public:
	// length of the list
	inline int Length() { return nodecount; };
	// List functions
	Filter *Push(const char *, Filter::Target &);
	void Pop();
	void Revoke(long Identifier);
	void Audit();
	bool IsPkgFiltered(pkgCache::PkgIterator);
	// Application operators
	void RedoLists(void);
	bool SimulateApply();
	// UI Helper Fuction
	void Display();
	FilterList(PackageTree *);
};

void filter_add_single(Screen *);
void filter_revoke_all(Screen *);
void filter_display_all(Screen *);
void filter_change_view(Screen *);

#endif

