/* $Id: pkgtree.cc,v 1.18 2001/05/15 08:32:58 ltd Exp $ */
#include <std.h>
#include <cstdio>
#include <ops.h>
#include <parsers/strutl.h>
#include <func/config.h>
#include <func/apt.h>
#include <func/tree.h>
#include <list/filters.h>
#include <list/screens.h>
#include <list/sections.h>
#include <interface/deityui.h>
#include <interface/format_strings.h>

// Begin "PackageTree" class functions

PackageTree::PackageTree(Screen *_Owner, ListType Init = AllPkgs) : Owner(_Owner)
{
	Sorting = new SortOrderList;
	Filters = new FilterList(this);

	root = 0;

	PackageCount = 0;

	Type = Init;

	if (Type == AllPkgs)
		inject_all_packages();
	else if (Type == ModPkgs)
		inject_changed_packages();
	
	RecalcLength();
}

PackageTree::~PackageTree()
{
	FreeTree(root);
}

void PackageTree::reinitialize()
{
	FreeTree(root);

	root = 0;
	PackageCount = 0;

	if (Type == AllPkgs)
		inject_all_packages();
	else if (Type == ModPkgs)
		inject_changed_packages();

	RecalcLength();
}

void PackageTree::apply_filters()
{
	reinitialize();
}

void PackageTree::inject_all_packages()
{
	INFO(INFO_DEBUG, "PackageTree::inject_all_packages()\n");

	pkgCache::PkgIterator I;
	pkgCache::VerIterator V;

	Item *P;

	pkgCache::Package ** SortedPkgs = new pkgCache::Package *[(*Cache)->Head().PackageCount + 1];
	Item **ParentList = new Item *[Sorting->DividerCount() + 1];
	memset(ParentList, 0, sizeof(Item *) * Sorting->DividerCount() + 1);
	Item *ThisParent = root;
	Item *LastItem = NULL;

	char *section_ptr = "/dev/null";
	char *priority_ptr = "/dev/null";
	const char *lastpkgname = "";
	char *new_section;
	int State = -100;
	const char *PA, *PB;
	char *a=0, *b=0, *p=0;

	register int J = 0;

	int ThisIncrement = 0;
	int H, ExpandedLevel = _Conf.GetInt("deity.tree.headers.expansion");
	bool HaveParent = false;

	if (Filters->Length() > 0)
	{
		for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
			if (Filters->IsPkgFiltered(I) == false)
				SortedPkgs[J++] = I;
	}
	else
	{
		for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
			SortedPkgs[J++] = I;
	}

	SortedPkgs[J] = 0;

	if (Sorting->Length() > 0)
		qsort(SortedPkgs, J, sizeof(*SortedPkgs), SortOrderList::qsort_handler);

	bool UseHeaders = _Conf.GetBool("deity.tree.headers.show");

	for (register int K = 0; K < J; K++)
	{
		pkgCache::PkgIterator curPkg(*Cache, SortedPkgs[K]);
 		V = curPkg.VersionList();

		if (V.end() == true)
			continue;

		if (UseHeaders == true)
		{
			H = 0;
			HaveParent = false;
			for (SortOrderList::SortIterator S = Sorting->ListBegin(); S.end() == false; S++, H++)
			{
				switch (S.method())
				{
				case SortOrderList::Section:
					new_section = (char *) curPkg.Section();
					INFO(INFO_DEBUG, "section = %s, new_section = %s\n", section_ptr, new_section);

					if (new_section == NULL)
						new_section = "";

					for (a = section_ptr + strlen(section_ptr); a >= section_ptr; a--)
						if (*a == '/')
						{
							a++;
							break;
						}
	
					if (*a == 0)
						a = section_ptr;

					for (b = new_section + strlen(new_section); b >= new_section; b--)
						if (*b == '/')
						{
							b++;
							break;
						}
	
					if (*b == 0)
						b = new_section;

					if (strcmp(a, b) != 0 || HaveParent)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Section);

						if (ThisIncrement == 0 || H == 0)
							P = add(&root);
						else
							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Sect;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;
	
				case SortOrderList::Status:
					if ((*Cache)[curPkg].Status != State || HaveParent)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Status);
	
						if (ThisIncrement == 0 || H == 0)
							P = add(&root);
						else
							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_State;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;
	
				case SortOrderList::Priority:
					PA = V.PriorityType();
					PB = priority_ptr;
					if (!PA) PA = "";
					if (!PB) PB = "";

					INFO(INFO_DEBUG, "priority A = %s, priority B = %s\n", PA, PB);
					
					if (strcmp(PA, PB) != 0)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Priority);
	
						if (ThisIncrement == 0 || H == 0)
							P = add(&root);
						else
							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Pri;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;
	
				case SortOrderList::Suite:
					a = strdup(curPkg.Section() ? curPkg.Section() : "");
					b = strdup(section_ptr ? section_ptr : "");

					if ((p = strchr(a, '/'))) *p=0;
					if ((p = strchr(b, '/'))) *p=0;

					if (strcasecmp(a, "non-us") != 0 && strcasecmp(a, "non-free") != 0 && strcasecmp(a, "contrib") != 0)
					{
						free(a);
						a = strdup("main");
					}

					if (strcasecmp(b, "non-us") != 0 && strcasecmp(b, "non-free") != 0 && strcasecmp(b, "contrib") != 0)
					{
						free(b);
						b = strdup("main");
					}

					if (strcasecmp(a, b) != 0 || HaveParent)
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::Suite);

						if (ThisIncrement == 0 || H == 0)
							P = add(&root);
						else
							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Custom;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						P->increment = ThisIncrement;
						P->Extra = strdup(a);

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}

					free(a); free(b);

					break;

				case SortOrderList::PkgName:
					if (*lastpkgname != *(curPkg.Name()))
					{
						ThisIncrement = Sorting->FindIncrement(SortOrderList::PkgName);
	
						if (ThisIncrement == 0 || H == 0)
							P = add(&root);
						else
							P = add(&ParentList[H - 1]->child, ParentList[H - 1]);

						P->TargetPkg = curPkg;
						P->Type = PackageTree::Item::Div_Custom;
						if (H < ExpandedLevel)
							P->State = PackageTree::Item::Expanded;
						char buf[2];
						*buf = *(curPkg.Name()); *(buf + 1) = 0;
						P->Extra = strdup(buf);
						P->increment = ThisIncrement;

						ThisParent = ParentList[H] = P;
						HaveParent |= true;
						LastItem = NULL;
					}
					break;

				default: ASSERT(0); break;
				}
			}
		}

		INFO(INFO_VERBOSE, "tree add pkg=%-30s sect=%s\n", curPkg.Name(), curPkg.Section());

		// Standard package! Woop.
		if (ThisIncrement == 0)
			P = add(&root, NULL, LastItem);
		else
			P = add(&ThisParent->child, ThisParent, LastItem);
		P->TargetPkg = curPkg;
		P->Type      = PackageTree::Item::Pkg;
		P->increment = ThisIncrement + 1;
		
		// Record previous priority
		priority_ptr = (char *) V.PriorityType();
		// Record previous section
		if (curPkg.Section() == NULL)
			section_ptr = "";
		else
			section_ptr = (char *) curPkg.Section();
		// Record previous inst state
		State = (*Cache)[curPkg].Status;
		// record last pkg name
		lastpkgname = curPkg.Name();
		// Record previous Item, speed up tree insert iterations
		LastItem = P;
	}

	delete [] SortedPkgs;
	delete [] ParentList;
}

void PackageTree::inject_changed_packages()
{
	pkgCache::PkgIterator I;

	Item *P, *Q;

	Q = add(&root);

	Q->increment = 0;
	Q->Type = PackageTree::Item::Div_Custom;
	Q->State = PackageTree::Item::Expanded;
	Q->Extra = (char *) strdup("Packages to Install");

	for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
	{
		if ((*Cache)[I].CandidateVerIter(*Cache).end() == true)
			continue;

		if ((*Cache)[I].NewInstall() == true)
		{
			P = add(&Q->child, Q);

			P->increment = 1;
			P->Type = PackageTree::Item::Pkg;
			P->TargetPkg = I;
			P->State = PackageTree::Item::None;
		}
	}

	Q = add(&root);

	Q->increment = 0;
	Q->Type = PackageTree::Item::Div_Custom;
	Q->State = PackageTree::Item::Expanded;
	Q->Extra = (char *) strdup("Packages to Reinstall");

	for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
	{
		if ((*Cache)[I].Delete() == false && ((*Cache)[I].iFlags & pkgDepCache::ReInstall) == pkgDepCache::ReInstall)
		{
			P = add(&Q->child, Q);

			P->increment = 1;
			P->Type = PackageTree::Item::Pkg;
			P->TargetPkg = I;
			P->State = PackageTree::Item::None;
		}
	}

	Q = add(&root);

	Q->increment = 0;
	Q->Type = PackageTree::Item::Div_Custom;
	Q->State = PackageTree::Item::Expanded;
	Q->Extra = (char *) strdup("Packages to Upgrade");

	for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
	{
		if ((*Cache)[I].Upgrade() == false || (*Cache)[I].NewInstall() == true)
			continue;

		if ((*Cache)[I].CandidateVerIter(*Cache).end() == true)
			continue;

		P = add(&Q->child, Q);

		P->increment = 1;
		P->Type = PackageTree::Item::Pkg;
		P->TargetPkg = I;
		P->State = PackageTree::Item::None;
	}

	Q = add(&root);

	Q->increment = 0;
	Q->Type = PackageTree::Item::Div_Custom;
	Q->State = PackageTree::Item::Expanded;
	Q->Extra = (char *) strdup("Packages to Delete");

	for (I = (*Cache)->PkgBegin(); I.end() != true; I++)
	{
		if ((*Cache)[I].CandidateVerIter(*Cache).end() == true)
			continue;

		if ((*Cache)[I].Delete() == false)
			continue;

		P = add(&Q->child, Q);

		P->increment = 1;
		P->Type = PackageTree::Item::Pkg;
		P->TargetPkg = I;
		P->State = PackageTree::Item::None;
	}
}

void PackageTree::FreeTree(Item *&z)
{
	if (z->child)
		FreeTree(z->child);
	
	if (z->next)
		FreeTree(z->next);

	remove(z);
}

PackageTree::Item *PackageTree::append(Item * pos)
{
	Item *ptr;

	if (pos == NULL)
		return NULL;

	ptr = pos->next;

	pos->next = (Item *) malloc(sizeof(Item));

	memset(pos->next, 0, sizeof(Item));

	pos->next->prev = pos;
	pos->next->next = ptr;
	pos->next->parent = pos->parent;
	pos->child = 0;

	if (ptr != NULL)
		ptr->prev = pos->next;

	PackageCount++;

	return pos->next;
}

PackageTree::Item *PackageTree::add(Item **head, Item *parent = 0, Item *last = 0, Item *carbon = 0)
{
	Item *P = 0;

	if (*head == 0)
	{
		*head = (Item *) malloc(sizeof(Item));

		if (carbon == 0)
			memset(*head, 0, sizeof(Item));
		else
			memcpy(*head, carbon, sizeof(Item));

		(*head)->parent = parent;
		(*head)->child = 0;
		(*head)->prev = 0;
		(*head)->next = 0;

		P = *head;
	}
	else
	{
		Item *Q;

		if (last)
			Q = last;
		else
			for (Q = *head; Q->next != 0; Q = Q->next);

		Q->next = (Item *) malloc(sizeof(Item));

		if (carbon == 0)
			memset(Q->next, 0, sizeof(Item));
		else
			memcpy(Q->next, carbon, sizeof(Item));
		
		Q->next->prev = Q;
		Q->next->next = 0;
		Q->next->parent = parent;
		Q->next->child = 0;

		P = Q->next;
	}

	PackageCount++;

	return P;
}

void PackageTree::remove(Item * p)
{
	Item *x, *y;

	if (p->IsDepend())
		delete static_cast<pkgCache::DepIterator *>(p->Extra);
	else if (p->Extra != NULL)
		free(p->Extra);

	x = p->next;
	y = p->prev;

	// Take care of our parent's child link if we are the first node in the child list
	if (p->parent != 0)
		if (p == p->parent->child)
			p->parent->child = p->next;

	// If we have children, make sure they are happy as well
	for (Item *z = p->child; z != 0; z = z->next)
			z->parent = p->parent;

	free(p);

	if (x != 0)
		x->prev = y;

	if (y != 0)
		y->next = x;

	if (p == root)
		root = 0;
	
	PackageCount--;
}

unsigned PackageTree::Locate(char *substring, unsigned long options)
{
	regex_t rectx;

	if (options & SEARCH_REGEX)
	{
		int reflags = REG_EXTENDED | REG_NOSUB;

		if (options & SEARCH_CASE)
			reflags |= REG_ICASE;

		regcomp(&rectx, substring, reflags);
	}
	else if (options & SEARCH_CASE)
	{
		strlower(substring);
	}

	unsigned numpkgsmatched = 0;
	const char *desc;
	bool DoExpansion = _Conf.GetBool("deity.search.expand_matched_item_parents");

	for (Iterator T(*this); !T.Invalid(); T.MoveNextLinear())
	{
		if (T->IsDivider())
			continue;

		if (options & SEARCH_DESC && T->Version().end() == false)
			desc = Cache->GetPkgRecs().Lookup(T->Version().FileList()).LongDesc().c_str();
		else
			desc = NULL;

		if (search_string(T->TargetPkg.Name(), substring, options, &rectx))
		{
			(int) T->State |= PackageTree::Item::Matched;

			if (DoExpansion)
				for (Item *x = T; x->parent; x = x->parent)
					(int) x->parent->State |= PackageTree::Item::Expanded;

			numpkgsmatched++;
		}
		else if (desc && search_string(desc, substring, options, &rectx))
		{
			(int) T->State |= PackageTree::Item::Matched;

			if (DoExpansion)
				for (Item *x = T; x->parent; x = x->parent)
					(int) x->parent->State |= PackageTree::Item::Expanded;

			numpkgsmatched++;
		}
	}

	if (options & SEARCH_REGEX)
		regfree(&rectx);

	return numpkgsmatched;
}

PackageTree::Item * PackageTree::LocateNext(bool start_at_beginning = false)
{
	Iterator T;

	switch (start_at_beginning)
	{
	case true:
		T = root;
		break;
	case false:
		T = Owner->cs;
		break;
	}

	for (; !T.Invalid(); T++)
	{
		if (T->IsDivider() || T == Owner->cs)
			continue;

		if (T->IsMatched())
			return T;
	}

	return NULL;
}

void PackageTree::RemoveState(int Z)
{
	for (Iterator I = root; !I.Invalid(); I++)
	{
		if (I->IsDivider())
			continue;

		(int) I->State &= ~Z;
	}
}

void PackageTree::CollapseDividers()
{
	for (Iterator I = root; !I.Invalid(); I.MoveNextLinear())
	{
		if (I->IsDivider())
			(int) I->State &= ~PackageTree::Item::Expanded;
	}
}

int PackageTree::CountState(int Z)
{
	int count = 0;

	for (Iterator I = root; !I.Invalid(); I++)
		if (I->State & Z)
			count++;

	return count;
}

string PackageTree::GetHeaderDesc(Item * P)
{
	string Format;
	char *ptr;
	const char *Section=NULL;

	if (P->IsDivider())
	{
		switch (P->Type)
		{
		case PackageTree::Item::Div_State:
			if ((*Cache)[P->TargetPkg].Status == 1)
				Format += FMT_DIV_STAT_INST;
			else if ((*Cache)[P->TargetPkg].Status == 0)
				Format += FMT_DIV_STAT_UPGR;
			else
				Format += FMT_DIV_STAT_NOTINST;
			break;

		case PackageTree::Item::Div_Sect:
			/*
			
			if (strncmp(P->TargetPkg.Section(), "contrib", 7) == 0)
			{
				if (strncmp(P->TargetPkg.Section() + 7, "/", 1) == 0)
				{
					Section = P->TargetPkg.Section() + 8;
					Format += "(contrib) ";
				}
				else
					Format += "(contrib) Contributed software packages";
			}
			else if (strncmp(P->TargetPkg.Section(), "non-free", 8) == 0)
			{
				if (strncmp(P->TargetPkg.Section() + 8, "/", 1) == 0)
				{
					Section = P->TargetPkg.Section() + 9;
					Format += "(non-free) ";
				}
				else
					Format += "(non-free) Software without a DFSG compatible license";
			}
			else if (strncasecmp(P->TargetPkg.Section(), "non-US", 6) == 0)
			{
				if (strncmp(P->TargetPkg.Section() + 6, "/", 1) == 0)
				{
					Section = P->TargetPkg.Section() + 7;
					Format += "(non-us) ";
				}
				else
					Format += "(non-us) Software that is restricted from being exported in the United States";
			}
			else
			{
				Section = P->TargetPkg.Section();
				Format += "(main) ";
			}
			*/

			if (P->TargetPkg.Section() == NULL)
				Format += FMT_DIV_SECT_NULL;
			else
			{
				Section = strrchr(P->TargetPkg.Section(), '/');
				if (Section == 0)
					Section = P->TargetPkg.Section();
				else
					Section += 1;
				Format += Section;
				Format += ": ";
				Format += Section_Lookup((char *)Section);
			}

			break;

		case PackageTree::Item::Div_Pri:

			ptr = (char *) (*Cache)[P->TargetPkg].CandidateVerIter(*Cache).PriorityType();

			if (ptr == 0 || *ptr == 0)
			{
				INFO(INFO_DEBUG, "Package %s has empty priority info\n", P->TargetPkg.Name());
				Format += FMT_DIV_PRI_NULL;
			}
			else
			{
				Format += "priority ";
				Format += ptr;
			}

			break;

		case PackageTree::Item::Div_Suite:
/*			ptr = strdup(P->TargetPkg.Section());
			x = strchr(ptr, '/');

			if (x == NULL)
				x = "main";
			else
				*x = 0;

			Format += ptr;

			free(ptr);
*/
			break;

		case PackageTree::Item::Div_Custom:

			Format += (char *) P->Extra;

			break;

		default:
			break;
		}
	}
	return Format;
}

void PackageTree::RecalcLength()
{
	int K = 0;
	for (Iterator I = root; I.Invalid() == false; I++, K++);
	PackageCount = K;
}
