/* $Id: pkgtree.h,v 1.6 2001/03/24 08:08:32 ltd Exp $ */
#ifndef _pkgtree_H
#define _pkgtree_H

#include <std.h>

#include <list/filters.h>
#include <list/sorting.h>

class FilterList;
class Screen;
class SortOrderList;

class PackageTree
{
	public:
	class Iterator;
	struct Item;
	friend Item;
	friend Iterator;
	friend FilterList;

	enum ListType { Empty = 0, AllPkgs = 1, ModPkgs = 2 };

protected:
	Screen * Owner;
	Item *root;
	ListType Type;
	long PackageCount;

	public:
	SortOrderList * Sorting;
	FilterList *Filters;

	// Methods to inject packages into the list (ListType)
	void reinitialize();
	void inject_all_packages();
	void inject_changed_packages();
	void FreeTree(Item *&);

	// Methods to filter the package list
	void apply_filters();

	// List modification functions
	Item *add(Item **head, Item *parent=NULL, Item *last=NULL, Item *carbon=NULL);
	Item *append(Item *prev=NULL);
	void remove(Item *);

	// Locate functions
	Item *LocateNext(bool start_at_beginning = false);
	Item *LocateFirst(void) { return LocateNext(true); };
	unsigned Locate(char *, unsigned long);

	// Flag typing functions
	void CollapseDividers();
	int CountState(int);
	void RemoveState(int);

	// Wrappers so that HiddenCount gets updated like it should
	void HidePkg(Item *);
	void ShowPkg(Item *);

	// Divider information
	string GetHeaderDesc(Item *);

	// Size operators
	inline long Length() { return PackageCount; };
	inline void RecalcLength();

	// Pointer accessors
	inline Item *Root() { return root; };

	// Hidden package seeking wrappers

	PackageTree(Screen *, ListType Init = AllPkgs);
	~PackageTree();
};

#include "pkgtreeitem.h"
#include "pkgtreeiterator.h"

#endif // _pkgtree_H
