/* $Id: sorting.cc,v 1.13 2001/05/15 08:32:58 ltd Exp $ 
 * This list is utilised when a PackageTree is constructed.
 */

#include <std.h>
#include <stdio.h>
#include <list/screens.h>
#include <func/config.h>
#include <func/apt.h>
#include <interface/deityui.h>
#include "sorting.h"

SortOrderList *LastOrderCreated = NULL;

SortOrderList::SortOrderList()
{
	Head = Tail = NULL;
	nodecount = 0;
	copy_from_config();
	LastOrderCreated = this;
}

SortOrderList::~SortOrderList()
{
	Flush();
}

void SortOrderList::Push(method_t myMethod)
{	
	nodecount += 1;

	if (Head == NULL && Tail == NULL)
	{
		Head = Tail = new Node;
		Tail->next = NULL;
	}
	else
	{
		Tail->next = new Node;
		Tail->next->next = NULL;
		Tail = Tail->next;
	}

	Tail->seqid = nodecount;
	Tail->thismethod = myMethod;

	reorderseqids();
}

void SortOrderList::Insert(SortIterator & S, method_t myMethod)
{	
	Node *x, *insertpos = S.Pos;

	if (insertpos == Head)
	{
		x = Head;
		Head = new Node;
		Head->next = x;

		x = Head;
	}
	else
	{
		for (x = Head; x->next != insertpos; x = x->next);

		Node *y = x->next;

		x->next = new Node;
		x->next->next = y;

		x = x->next;
	}
		
	x->seqid = 0;
	x->thismethod = myMethod;

	nodecount += 1;

	reorderseqids();
}

void SortOrderList::Pop(void)
{
	struct Node *p;

	nodecount -= 1;

	if (Head == Tail)
	{
		delete Head;
		Head = Tail = NULL;
	}
	else
	{
		for (p = Head; p->next != Tail; p = p->next);
		p->next = p->next->next;
		delete Tail;
		Tail = p;
	}

	reorderseqids();
}

void SortOrderList::remove(SortIterator & S)
{
	Node *p = S.Pos;

	Node *x = NULL;

	if (p == Head && p == Tail)
	{
		Head = Tail = NULL;
		delete p;
		nodecount = 0;
	}
	else if (p == Head)
	{
		x = Head;
		Head = Head->next;
		delete x;
		nodecount -= 1;
	}
	else if (p == Tail)
	{
		for (x = Head; x->next != Tail; x = x->next);
		x->next = x->next->next;
		delete Tail;
		Tail = x;
		nodecount -= 1;
	}
	else
	{
		for (x = Head; x->next != p; x = x->next);
		x->next = x->next->next;
		delete p;
		nodecount -= 1;
	}

	reorderseqids();
}

void SortOrderList::Flush(void)
{
	while (nodecount > 0)
		Pop();
}


/*
 * Useful for formatting dividers ... [Order of sort methods]
 */
int SortOrderList::FindIncrement(enum SortOrderList::method_t Type)
{
	for (SortIterator S = ListBegin(); S.end() == false; S++)
	{
		if (S.method() == Type)
			return S.seq()-1;
	}

	return -1;
}

/*
 * Useful for formatting dividers ... [Order of sort methods]
 */
bool SortOrderList::Exists(enum SortOrderList::method_t Type)
{
	for (SortIterator S = ListBegin(); S.end() == false; S++)
	{
		if (S.method() == Type)
			return true;
	}

	return false;
}

/*
 * Simple.. count the number of sort methods that divide the package list
 */
int SortOrderList::DividerCount()
{
	int I = 0;

	for (SortIterator S = ListBegin(); S.end() == false; S++)
	{
		if (S.method() == Status || S.method() == Section || S.method() == Priority)
			I++;
	}

	return I;
}

void SortOrderList::reorderseqids()
{
	int I;
	SortIterator S;

	for (I = 1, S = ListBegin(); S.end() == false; S++, I++)
		S.Pos->seqid = I;
}

void SortOrderList::commit_to_config()
{
	string s;
	char buf[50];

	for (SortIterator S = ListBegin(); S.end() == false; S++)
	{
		sprintf(buf, "%d", S.method());
		s += buf;
		s += "/";
	}

	if (nodecount > 0)
		s.resize(s.size()-1);
	else
		s = "NULL";

	_Conf.Set("deity.sorting", (char *)s.c_str());
}

void SortOrderList::copy_from_config()
{
	Flush();

	char *tmpbuf = strdup(_Conf.Get("deity.sorting"));

	if (strcmp(tmpbuf, "NULL") != 0)
		for (char *p = strtok(tmpbuf, "/"); p != NULL; p = strtok(NULL, "/"))
			Push((SortOrderList::method_t)atoi(p));

	free(tmpbuf);
}

SortOrderList::SortIterator SortOrderList::ListFind(method_t ThisMethod)
{
	SortIterator S(ListBegin());

	for (; S.end() == false; S++)
		if (S.method() == ThisMethod)
			return S;

	return S;
}

int SortOrderList::qsort_handler(const void *a, const void *b)
{
	const pkgCache::Package & A = **(pkgCache::Package **) a;
	const pkgCache::Package & B = **(pkgCache::Package **) b;
	pkgCache::PkgIterator IterA(*Cache, *(pkgCache::Package **) a);
	pkgCache::PkgIterator IterB(*Cache, *(pkgCache::Package **) b);
	pkgCache::VerIterator VerA = IterA.VersionList();
	pkgCache::VerIterator VerB = IterB.VersionList();

	SortIterator S;
	SortIterator End = LastOrderCreated->ListEnd();

	for (S = LastOrderCreated->ListBegin(); S.end() == false; S++)
	{
		if (S == End)
		{
			switch (S.method())
			{
			case None:
				break;
			case Section:
				return compare_section(&A, &B);
			case PkgName:
				return compare_package_name(&A, &B);
			case Priority:
				return compare_priority(VerA, VerB);
			case Status:
				return compare_status(IterA, IterB);
			case Size:
				return compare_archive_size(VerA, VerB);
			case Instsize:
				return compare_installed_size(VerA, VerB);
			case Distribution:
				return compare_distribution(VerA, VerB);
			case Suite:
				return compare_suite(&A, &B);
			}
		}
		else
		{
			int comparison = 0;
			
			switch (S.method())
			{
			case None:
				break;
			case Section:
				comparison=compare_section(&A, &B);
				break;
			case PkgName:
				comparison=compare_package_name(&A, &B);
				break;
			case Priority:
				comparison=compare_priority(VerA, VerB);
				break;
			case Status:
				comparison=compare_status(IterA, IterB);
				break;
			case Size:
				comparison=compare_archive_size(VerA, VerB);
				break;
			case Instsize:
				comparison=compare_installed_size(VerA, VerB);
				break;
			case Distribution:
				comparison=compare_distribution(VerA, VerB);
				break;
			case Suite:
				comparison=compare_suite(&A, &B);
				break;
			}

			if (comparison != 0)
				return comparison;
		}
	}

	// This shouldn't happen

	return 0;
}

int SortOrderList::compare_section(const pkgCache::Package * A, const pkgCache::Package * B)
{
	char *a, *b;
	char *strp = Cache->GetCache()->StrP;

	for (a = strp + A->Section; *a != 0; a++)
		if (*a == '/')
		{
			a++;
			break;
		}

	if (*a == 0)
		a = strp + A->Section;

	for (b = strp + B->Section; *b != 0; b++)
		if (*b == '/')
		{
			b++;
			break;
		}
	
	if (*b == 0)
		b = strp + B->Section;
	
	return strcmp(a, b);
}

int SortOrderList::compare_package_name(const pkgCache::Package * A, const pkgCache::Package * B)
{
	char *strp = Cache->GetCache()->StrP;
	return strcmp(strp + A->Name, strp + B->Name);
}

int SortOrderList::compare_priority(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB)
{
	const char *PA = VerA.PriorityType(), *PB = VerB.PriorityType();

	if (PA == 0)
		PA = "";
	if (PB == 0)
		PB = "";

	return strcmp(PA, PB);

	if (strcmp(PB, "Required") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return 0;
		else if (strcmp(PA, "Important") == 0)
			return 1;
		else if (strcmp(PA, "Standard") == 0)
			return 2;
		else if (strcmp(PA, "Optional") == 0)
			return 3;
		else if (strcmp(PA, "Extra") == 0)
			return 4;
		else
			return 5;
	}
	else if (strcmp(PB, "Important") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return -1;
		else if (strcmp(PA, "Important") == 0)
			return 0;
		else if (strcmp(PA, "Standard") == 0)
			return 1;
		else if (strcmp(PA, "Optional") == 0)
			return 2;
		else if (strcmp(PA, "Extra") == 0)
			return 3;
		else
			return 4;
	}
	else if (strcmp(PB, "Standard") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return -2;
		else if (strcmp(PA, "Important") == 0)
			return -1;
		else if (strcmp(PA, "Standard") == 0)
			return 0;
		else if (strcmp(PA, "Optional") == 0)
			return 1;
		else if (strcmp(PA, "Extra") == 0)
			return 2;
		else
			return 3;
	}
	else if (strcmp(PB, "Optional") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return -3;
		else if (strcmp(PA, "Important") == 0)
			return -2;
		else if (strcmp(PA, "Standard") == 0)
			return -1;
		else if (strcmp(PA, "Optional") == 0)
			return 0;
		else if (strcmp(PA, "Extra") == 0)
			return 1;
		else
			return 2;
	}
	else if (strcmp(PB, "Extra") == 0)
	{
		if (strcmp(PA, "Required") == 0)
			return -4;
		else if (strcmp(PA, "Important") == 0)
			return -3;
		else if (strcmp(PA, "Standard") == 0)
			return -2;
		else if (strcmp(PA, "Optional") == 0)
			return -1;
		else if (strcmp(PA, "Extra") == 0)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}

int SortOrderList::compare_status(pkgCache::PkgIterator & IterA, pkgCache::PkgIterator & IterB)
{
	if ((*Cache)[IterA].Status < (*Cache)[IterB].Status)
		return -1;
	else if ((*Cache)[IterA].Status > (*Cache)[IterB].Status)
		return 1;
	else
		return 0;
}

int SortOrderList::compare_archive_size(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB)
{
	return VerA->Size > VerB->Size;
}

int SortOrderList::compare_installed_size(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB)
{
	return VerA->InstalledSize > VerB->InstalledSize;
}

int SortOrderList::compare_distribution(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB)
{
	return 0;
}

int SortOrderList::compare_suite(const pkgCache::Package * A, const pkgCache::Package * B)
{
	char *a, *b, *p;
	char *strp = Cache->GetCache()->StrP;
	
	a = strdup(strp + A->Section);
	b = strdup(strp + B->Section);

	if ((p = strchr(a, '/')))
		*p = 0;
	if ((p = strchr(b, '/')))
		*p = 0;

#define RET(x) { free(a); free(b); return(x); }

	if (strcasecmp(b, "non-us") == 0)
	{
		if (strcasecmp(a, "non-us") == 0)
			RET(0)
		else if (strcasecmp(a, "non-free") == 0)
			RET(1)
		else if (strcasecmp(a, "contrib") == 0)
			RET(2)
		else
			RET(3)
	}
	else if (strcasecmp(b, "non-free") == 0)
	{
		if (strcasecmp(a, "non-us") == 0)
			RET(-1)
		else if (strcasecmp(a, "non-free") == 0)
			RET(0)
		else if (strcasecmp(a, "contrib") == 0)
			RET(1)
		else
			RET(2)
	}
	else if (strcasecmp(b, "contrib") == 0)
	{
		if (strcasecmp(a, "non-us") == 0)
			RET(-2)
		else if (strcasecmp(a, "non-free") == 0)
			RET(-1)
		else if (strcasecmp(a, "contrib") == 0)
			RET(0)
		else
			RET(1)
	}
	else
	{
		if (strcasecmp(a, "non-us") == 0)
			RET(-3)
		else if (strcasecmp(a, "non-free") == 0)
			RET(-2)
		else if (strcasecmp(a, "contrib") == 0)
			RET(-1)
		else
			RET(0)
	}
}
	

