/* $Id: sorting.h,v 1.9 2000/11/17 14:56:21 ltd Exp $ */
#ifndef _sort_h
#define _sort_h

#include <func/apt.h>

class SortOrderList
{
	public:
		class SortIterator;
		friend SortIterator;

		struct Node;

		// Define the different sort methods that can be added to this methodlist
		enum method_t { 
			None=0, 
			Section=1, 
			PkgName=2, 
			Priority=3, 
			Status=4, 
			Size=5, 
			Instsize=6,
			Distribution=7,
			Suite=8 };

		// length of the list
		inline int Length() { return nodecount; };
		
		// List functions
		void Push(method_t);
		void Insert(SortIterator &, method_t);
		void Pop();
		void remove(SortIterator &);
		void Flush();

		static int compare_section(const pkgCache::Package * A, const pkgCache::Package * B);
		static int compare_package_name(const pkgCache::Package * A, const pkgCache::Package * B);
		static int compare_priority(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB);
		static int compare_status(pkgCache::PkgIterator & IterA, pkgCache::PkgIterator & IterB);
		static int compare_archive_size(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB);
		static int compare_installed_size(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB);
		static int compare_distribution(pkgCache::VerIterator & VerA, pkgCache::VerIterator & VerB);
		static int compare_suite(const pkgCache::Package * A, const pkgCache::Package * B);
		static int qsort_handler(const void *, const void *);

		// Inquisitory functions
		int DividerCount();
		int FindIncrement(enum method_t);
		bool Exists(enum method_t);

		void reorderseqids();
		void commit_to_config();
		void copy_from_config();

		// Iterator thingies
		SortIterator ListBegin();
		SortIterator ListEnd();
		SortIterator ListFind(method_t);

		// con/destructers
		SortOrderList();
		~SortOrderList();

	protected:
		unsigned int nodecount;
		Node *Head;
		Node *Tail;
};

struct SortOrderList::Node
{
	unsigned int seqid;
	enum SortOrderList::method_t thismethod;

	struct SortOrderList::Node *next;
};

class SortOrderList::SortIterator
{
	public:
		friend SortOrderList;
	
		inline void operator =(SortIterator B) { Pos = B.Pos; };
		void operator ++(int) { Pos = Pos->next; };
		inline void operator ++() {operator ++(0);};

		inline bool operator ==(SortIterator &B) { return Pos == B.Pos; };
		inline bool operator !=(SortIterator &B) { return Pos != B.Pos; };

		inline bool end() { return Pos == 0; };

		inline enum method_t method() { return Pos->thismethod; };
		inline unsigned int seq() { return Pos->seqid; };

		inline SortIterator(SortOrderList &Owner) { Pos = Owner.Head; };
		inline SortIterator(Node *NewPos) { Pos = NewPos; };
		inline SortIterator() {};

	protected:
		Node *Pos;
};

inline SortOrderList::SortIterator SortOrderList::ListBegin() { return SortIterator(Head); }
inline SortOrderList::SortIterator SortOrderList::ListEnd() { return SortIterator(Tail); }

extern SortOrderList *LastOrderCreated;

#endif

